/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* DRAG RACE TIMER RACE MANAGER                                                    Version 14  01/18/2024 *
* Created by: W. van Leeuwen (Billv923@outlook.com)                                                      *
*                                                                                                        *
* This sketch provides the display and race management interface for a 2-Lane 1/4 mile Drag Race Timer.  *
* It works in conjunction with the drag race timer software resident on the Arduino based timer unit.    *
* The Arduino timer unit provides the hardware interfaces for the start line staging sensors, the 60Ft,  *
* 660Ft, 1254Ft & 1320 Ft down track sensors and the interface to drive the Christmas Tree lights. The   *
* timer unit's software performs track management functions, makes the track time measurements & speed   *
* calculations, determines win/lose status and controls the cycling of the Christmas Tree lights. Race   *
* data is sent to the PC running this sketch via the serial (USB) interface.                             *
*                                                                                                        *
* NOTE - This version operates in conjunction with Arduino software DRT_Timer_Ver14.ino                  *
*                                                                                                        *
* This version provides the following features:                                                          *
*   1. Select & Display of Racer's Class, Car#, & Dial-In times from a roster                            *
*   2. Display of Reaction Time, 60Ft ET, 660 Ft (1/8 Mile), 1320Ft (1/4 Mile) ET & Speed, Margin and    *
*      race status (Win, Draw or Foul)                                                                   *
*   3. Enable/Disable "Double Red" and "Top Bulb Lock" options                                           *
*   4. Ability to select between Standard, Pro or Street Outlaw Xmas Tree functionality                  *
*   5. Ability to select between manual race or auto race start                                          *
*   6. Ability to set timer to single lane (Lane1 or Lane2 selectable) or dual lane operation            *
*   7. Abiltiy to perform Handicapped (AKA Bracket) racing                                               *
*   8. Race duration counter to display number of seconds elapsed while the race is in progress          *
*   9. Automatically save race results to a comma-delimited (.csv) file                                  *
*  10. Capability to load and continue from a previously saved race event                                *
*  11. Generate and print racer's time slips                                                             *
*  12. Process timer reset & race start requests from remote hand-held pushbutton                        *
*  13. Track status monitoring & display                                                                 *
*  14. Displays a negative reaction time in red when racer red-lights (Fouls) to show how early he left  *
*      the start line.                                                                                   *
*  15. Ability to command the Arduino timer to initiate the Christmas Tree lamp test (Key F1)            *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
   
import processing.serial.*;                         //Use the serial library
import static javax.swing.JOptionPane.*;            //Use the option pane dialog box library

/* * * * * * * * * * * * * * * * * * * DEFAULT USER MODIFIABLE PARAMETERS * * * * * * * * * * * * * * * * * * */
boolean MetricFlag = false;                         //Set to true when speed is in Km/h
String Disp_Title = "Organization Name Goes Here";  //Text string to hold display title
String TS_Header1 = "DFW Winternationals";          //Time Slip header line 1 (33 Char Max)
String TS_Header2 = "Championship Drag Racing";     //Time Slip header line 2 (33 Char Max)
String TS_Footer1 = "Thanks for racing with us";    //Time Slip footer line 1 (33 Char Max)
String TS_Footer2 = "Come back soon";               //Time Slip footer line 2 (33 Char Max)
int Autoprintcopies = 0;                            //Number of copies to automatically print after race
int timeout = 20;                                   //Race timeout in seconds (32 Max)
//Set the default file path for the Time Slip & Parameters file - MUST BE SAME FILE PATH AS THIS FILE
String filePath = "C:/Users/Bill/Timers/Drag Race Timers/DRT_Ver14_With_Roster/DRT_RaceManager_Ver14/";
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

// Initialization
  int i, j, k, n, x, y;                     //working integer variables
  int Rx, Gx, Bx;                           //Background color RGB values
  int Yr, Mo, Day, Hr, Min;                 //Integer variables for creating date-time based filename
  int Sec;                                  //Clock seconds variable for use by the Race in Progress counter
  int Counter = 0;                          //Variable for holding race in progress duration (seconds)
  int Lane_Usage = 3;                       //Used to determine which lanes are racing 1=lane1, 2=Lane2, 3=Lane1&2
  int Race_State = 0;                       //0=Not Ready, 1=Ready, 2=Staged, 3=Countdown, 4=Racing, 5=Finished
  int Msg_Number = 0;                       //0=None, 1=Msg1, 2=Msg2, 3=Msg3, 4=Msg4, etc
  int Race_Number = 0;                      //Race number selected/displayed
  int Races_Ran = 0;                        //Number of races ran
  int Max_Races = 999;                      //Maximum number of races allowed
  int Tablerow = -1;                        //Results table row index pointer
  int lf = 10;                              //ASCII Linefeed
  int Index1;                               //Race results table index pointer
  int RosterIndex;                          //Roster table index pointer
  int Ln1Foul = 0;                          //Lane1 foul flag (1=fouled 1'st, 2=fouled 2'nd)
  int Ln2Foul = 0;                          //Lane2 foul flag (1=fouled 1'st, 2=fouled 2'nd)
  int Brkout1flag = 0;                      //Lane1 breakout flag
  int Brkout2flag = 0;                      //Lane2 breakout flag
  int Win1flag = 0;                         //Lane1 win flag
  int Win2flag = 0;                         //Lane2 win flag
  int L1Color;                              //Used for holding color value for Lane1 status
  int L2Color;                              //Used for holding color value for Lane2 status
  int [] Place = new int[2];                //Array for storing lane finish order (2 lanes)
  boolean ComPort_Avail = false;            //Denotes when com port is available. Default to false
  boolean ComSelectedflag = false;          //Denotes when com port has been selected
  boolean Canceledflag = false;             //Used to determine if file selection was canceled
  boolean PracticeRaceflag = false;         //Indicates when practice run data is available to print
  boolean NextRaceflag = true;              //Indicates when Next Race button is to be active
  boolean Racefile_Selected = false;        //Indicates when a previously saved race event file has been selected
  boolean RaceContinueflag = false;         //Indicates continuation of previously saved race event
  boolean Holdflag = false;                 //Set to true when viewing prior ran race results
  boolean NoComflag = false;                //Set when no COM ports are detected in setup routine
  boolean Autostartflag = false;            //Set to true when timer auto start is selected
  boolean SetupPgSelect = true;             //Set to true to display the Setup Page
  boolean RosterFile_Selected = false;      //Denotes when user has selected a Roster file to use
  boolean RosterLoadedflag = false;         //Denotes when user has loaded a roster file
  boolean DoubleRedflag = false;            //Set when Double red mode is enabled
  boolean TopBulbLockflag = false;          //Set when Top Bulb Lock mode is enabled
  boolean HandiCapEnblflag = true;          //Set to false when to ignore racer dial-in times
  boolean IndyTSflag = false;               //True = individual timeslips
  boolean AddDriverflag = false;            //Used in roster add name routine
  float [] DialinTime = new float[2];       //Array for storing racer's dial-in time values (2 lanes)
  float [] ReactTime = new float[2];        //Array for storing racer's reaction time values (2 lanes)
  float [] Trk60ftTime = new float[2];      //Array for storing racer's 60 Foot ET values (2 lanes)
  float [] Trk660ftTime = new float[2];     //Array for storing racer's 660 Foot ET values (2 lanes)
  float [] Trk1320ftTime = new float[2];    //Array for storing racer's 1320 Foot ET values (2 lanes)
  float [] OverallTime = new float[2];      //Array for storing racer's overall race time values (2 lanes)
  float [] LaneET = new float[2];           //Array for storing racer's 1/8 or 1/4 mile ET values (2 lanes)
  float [] Margin = new float[2];           //Array for storing racer's overall race time values (2 lanes)
  float [] Speed = new float[2];           //Array for storing racer's 1/4 Mile speed (2 lanes)
  float Value;                              //Working Floating point variable
  String [] RClass = new String[2];         //Lane1 & Lane2 variables for holding race class
  String [] Carnum = new String[2];         //Lane1 & Lane2 variables for holding car number 
  String Str1, Str2, Str3 = "";             //Working string variables
  String COMx, COMlist = "";                //COM Port check/select routine variables
  String Portname = "";                     //COM Port name variable
  String Datafile = "";                     //String variable for the save data file name
  String TSfileName="Timeslip.txt";         //String variable for the Time Slip text file name
  String RosterFilename = "Roster.csv";     //String variable for the User's Roster data file name
  String parmFilename="User_Parameters.csv"; //String variable user modifiable parameters file
  String parmFilepath = "";                 //Used to store file path/name of the user parameters file
  String raceFilename = "";                 //Used to store file path/name of selected file
  String TS_date="";                        //String variable to hold time Slip date stamp
  String TS_time="";                        //String variable to hold time Slip time stamp
  String Time_String = "";                  //String variable for temp storage of race time values
  String Tree_Mode = "M:0";                 //M:0=Standard mode, M:1=Pro mode, M:2=Outlaw mode
  String Msg1 = "";                         //String variable to hold lane sensor error message
  String Msg1a = "Sensor(s)";
  String Msg1b = " not ready. Fix problem & click Timer Reset";
  String Msg2 = "Hand-held race start switch not ready. Fix problem & click Timer Reset";
  String Msg3 = "Click Next Race button to prep for next race";
  String Msg4 = "Click Timer Reset to run a practice race or Next Race button to prep for first official race";
  String Msg5 = "Remote RESET detected";
  String Msg6 = "Remote START detected";
  String Msg7 = "OPTIONAL: Select & assign Lane 1 & 2 Drivers from the Roster";
  String prompt = "";                       //Variable used for storing popup message text
  String L1status = "";                     //Variable for holding Lane1 race status
  String L2status = "";                     //Variable for holding Lane2 race status
  String inString = "";                     //Input string from serial port
  String Prt_Cmd = "";                      //String for holding print command
  String [] TSParam = new String[] {" Class  ","  Car#  ","  R/T   "," 60 Ft  "," 1/8 ET ",
         " 1/4 ET ","1/4 MPH "," Status "," Margin "};
  PFont font;
  Table Rostertable;                        //Define table named 'Rostertable' to hold racer info
  Table Resultstable;                       //Define table named 'Resultstable' to hold race results
  Table Parmtable;                          //Define table named 'Parmtable' to hold user parameters
  PrintWriter TS_output;                    //Create an output variable for writing to the Timeslip.txt file
  PImage logo;                              //Define an image variable as 'logo'
 
  Serial myPort;                            //Set the Serial port
  
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void setup() {
  size(1200, 700);                                 //Set display area in pixels
  logo = loadImage("BillV_Logo.png");              //Load "Bill V's Derby Timers" Logo
  Prt_Cmd = "notepad.exe /p "+filePath+TSfileName; //Set the time slip print command string
  
  //Create unique date-time based file name so that a new file is created each time the program is run.
  //This will prevent previous files from being accidentally overwritten. Note that by default the file
  //is saved to the same folder/directory that contains this sketch.
  Yr = year();                              //Get year value from PC clock
  Mo = month();                             //Get month value from PC clock
  Day = day();                              //Get day value from PC clock
  Hr = hour();                              //Get hour value from PC clock
  Min = minute();                           //Get minute value from PC clock
  Datafile = "Race_Results_"+nf(Yr,4)+nf(Mo,2)+nf(Day,2)+nf(Hr,2)+nf(Min,2)+".csv";
  
  //Initialize the user modifiable parameters table. The table is used to import user modifiable parameters
  Parmtable = new Table();                  //Name the new table
  Parmtable.addColumn("PARAMETER");         //Add a column named PARAMETER for parameter name
  Parmtable.addColumn("VALUE");             //Add a column named VALUE for parameter valuee
  
  //Load user parameters
  parmFilepath = filePath + parmFilename;
  File Filename = new File(parmFilepath);
  if (Filename.exists()) {
    Parmtable = loadTable(parmFilepath, "header");  //Load user modifiable parameters table
    Disp_Title = Parmtable.getString(0, "VALUE");   //Get display title
    TS_Header1 = Parmtable.getString(1, "VALUE");   //Get TS header line1
    TS_Header2 = Parmtable.getString(2, "VALUE");   //Get TS header line2
    TS_Footer1 = Parmtable.getString(3, "VALUE");   //Get TS footer line1
    TS_Footer2 = Parmtable.getString(4, "VALUE");   //Get TS footer line1
    Autoprintcopies = Parmtable.getInt(5, "VALUE"); //Get TS Auto-print copy count
    timeout = Parmtable.getInt(6, "VALUE");         //Get timeout value (seconds)
    Str1 = Parmtable.getString(7, "VALUE");         //Get Metric flag state
    if ((Str1.indexOf("T")>=0)) MetricFlag = true;  //Set Metric flag to true
      else MetricFlag = false;                      //Set Metric flag to false
  }
  else {
    prompt = "          * * * WARNING * * *\nUser Parameter File: Parameters.csv\nNot Found. Default values will be used";
    showMessageDialog(null,prompt);         //Display warning message
  }
  
  //Initialize the 'Rostertable'. The table is used to store racer data
  Rostertable = new Table();                //Name the new table
  Rostertable.addColumn("Name");            //Add a column for racer's name
  Rostertable.addColumn("CarNo");           //Add a column for racer's car#
  Rostertable.addColumn("Class");           //Add a column for racer's class
  Rostertable.addColumn("Dialin");          //Add a column for racer's dial-in time
  
  //Initialize the 'Resultstable'. The table is used to save data from each race to a data file
  Resultstable = new Table();               //Name the new table
  Resultstable.addColumn("Race");           //Add a column named Race for Race Number
  Resultstable.addColumn("L1_Car");         //Add a column for Lane1 Car#
  Resultstable.addColumn("L1_Class");       //Add a column for Lane1 Class
  Resultstable.addColumn("L1_Foul");        //Add a column for Lane1 Foul status
  Resultstable.addColumn("L1_BrkOutflag");  //Add a column for Lane1 Breakout flag
  Resultstable.addColumn("L1_Dialin");      //Add a column for Lane1 Dial-in Time
  Resultstable.addColumn("L1_React");       //Add a column for Lane1 Reaction Time
  Resultstable.addColumn("L1_60Ft");        //Add a column for Lane1 60 Ft time
  Resultstable.addColumn("L1_660Ft");       //Add a column for Lane1 660 Ft time
  Resultstable.addColumn("L1_1320Ft");      //Add a column for Lane1 1/4 Mile Time
  Resultstable.addColumn("L1_Speed");       //Add a column for Lane1 1/4 mile Speed
  Resultstable.addColumn("L1_Ovl");         //Add a column for Lane1 overall time
  Resultstable.addColumn("L1_Margin");      //Add a column for Lane1 Margin
  Resultstable.addColumn("L1_Winflag");     //Add a column for Lane1 Win flag
  Resultstable.addColumn("L2_Car");         //Add a column for Lane2 Car#
  Resultstable.addColumn("L2_Class");       //Add a column for Lane2 Class
  Resultstable.addColumn("L2_Foul");        //Add a column for Lane2 Foul status
  Resultstable.addColumn("L2_BrkOutflag");  //Add a column for Lane2 Breakout flag
  Resultstable.addColumn("L2_Dialin");      //Add a column for Lane2 Dial-in Time
  Resultstable.addColumn("L2_React");       //Add a column for Lane2 Reaction Time
  Resultstable.addColumn("L2_60Ft");        //Add a column for Lane2 60 Ft time
  Resultstable.addColumn("L2_660Ft");       //Add a column for Lane2 660 Ft time
  Resultstable.addColumn("L2_1320Ft");      //Add a column for Lane2 1/4 Mile Time
  Resultstable.addColumn("L2_Speed");       //Add a column for Lane2 1/4 mile Speed
  Resultstable.addColumn("L2_Ovl");         //Add a column for Lane2 overall time
  Resultstable.addColumn("L2_Margin");      //Add a column for Lane2 Margin
  Resultstable.addColumn("L2_Winflag");     //Add a column for Lane2 Win flag
  Resultstable.addColumn("Date");           //Add a column for race date
  Resultstable.addColumn("Time");           //Add a column for race time
  
  //Initialize other parameters
  for (i=0; i<2; i++) {
    RClass[i] = " ";
    Carnum[i] = " ";
  }
  
  SetupPgSelect = true;                     //Set page selection to Setup page
  Init_Setup_Page();                        //Call subroutine to initialize the setup page
} //End of "Setup" routine

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                           MAIN ROUTINE                                               *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void draw() {  //Note: Things aren't displayed on the screen until the draw loop completes.
  if (ComPort_Avail && !SetupPgSelect) {       //Do if Race page is selected
    //GET & PROCESS SERIAL DATA FROM TIMER
    if (inString.indexOf("NRD") >= 0) {        //Check for receipt of 'Not Ready' state message
      Race_State = 0;                          //Set the race state to 0 (0=not ready)
      Set_Race_State();                        //and call subroutine to display the race state
      if (Race_Number > 0) {                   //Don't allow if no races have been ran yet
        NextRaceflag = false;                  //Disable Next Race button
        Draw_NextRace_Button();                //Call subroutine to update Next Race button display
      }
      Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
      inString = "";                           //Clear 'Not Ready' message once processed
    }
    if (inString.indexOf("RDY") >= 0) {        //Check for receipt of 'Ready' state
      Race_State = 1;                          //Set the race state to 1 (1=ready)
      Clear_Data();                            //Call subroutine to clear data from text boxes
      Clear_Msgbox();                          //Call subroutine to initialize/clear Message box
      Carnum[0] = Carnum[1] = " ";             //Clear racer car numbers
      RClass[0] = RClass[1] = " ";             //Clear racer class names
      DialinTime[0] = DialinTime[1] = 0.0;     //Clear Dial-in times
      Counter = j = 0;                         //Clear the race duration counter
      Set_Race_State();                        //Call subroutine to display the race state
      if (Race_Number > 0) {                   //Don't allow if no races have been ran yet
        NextRaceflag = false;                  //Disable Next Race button
        Draw_NextRace_Button();                //Call subroutine to update Next Race button display
      }
      inString = "";                           //Clear 'Ready' message once processed
      Disp_Message(7);                         //Call subroutine to display Racer ID entry message
    }
    if (inString.indexOf("STG") >= 0) {        //Check for receipt of 'Staged' state
      Race_State = 2;                          //Set the race state to 2 (2=staged)
      Set_Race_State();                        //and call subroutine to display the race state
      Clear_Msgbox();                          //Call subroutine to clear Message box
      Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
      inString = "";                           //Clear 'Staged' message once processed
    }
    if (inString.indexOf("CNT") >= 0) {        //Check for receipt of 'Countdown' state
      Race_State = 3;                          //Set the race state to 3 (3=countdown)
      Set_Race_State();                        //and call subroutine to display the race state
      Clear_Msgbox();                          //Call subroutine to clear Message box
      Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
      inString = "";                           //Clear 'Countdown' message once processed
    }
    if (inString.indexOf("RAC") >= 0) {        //Check for receipt of 'Racing' state
      Race_State = 4;                          //Set the race state to 4 (4=racing)
      Set_Race_State();                        //and call subroutine to display the race state
      Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
      inString = "";                           //Clear 'Racing' message once processed
    }
    if (inString.indexOf("TRK") >= 0) {        //Check for receipt of Track Sensor error code
      i = inString.length() - 2;               //Get length of inString minus cr/lf characters
      Str1 = inString.substring(3, i);         //Extract lane sensor ID's from inString
      Msg1 = Msg1a + Str1 + Msg1b;             //Construct the error message to display
      Disp_Message(1);                         //Call subroutine to display track sensor error message
      inString = "";                           //Clear error message once processed
    }
    if (inString.indexOf("RST") >= 0) {        //Check for receipt of Remote RESET message
      Disp_Message(5);                         //Call subroutine to display Remote Reset message
      if (NextRaceflag && (Races_Ran<=Max_Races)) { //Only allow if next race flag is TRUE & races ran <= 999
        Race_Number = Races_Ran + 1;           //Set the current race number
        Draw_Racebox();                        //Redisplay new race value
      }
      inString = "";                           //Clear reset message once processed
    }
    if (inString.indexOf("Lane1") >= 0) {      //Check for receipt of Lane1 Race times
      Time_String = inString;                  //and save it to a temp holding variable
      //Format: Lane1: x y aaaa.aaa bbbb.bbb cccc.ccc dddd.ddd eeee.eee z
      //Where: x=Foul flag, y=Breakout flag, aaaa.aaa=RT, bbbb.bbb=60Ft ET, cccc.ccc=660Ft ET, 
      //       dddd.ddd==1320Ft ET, eeee.eee=1320Ft Speed, z=Win flag
      Str1 = Time_String.substring(7, 8);      //Extract foul status for Lane1
      Ln1Foul = 0;                             //Preset Lane1 foul status to not fouled
      if (Str1.equals("1")) Ln1Foul = 1;       //And set Lane1 foul status to 1 if 1'st lane to foul
      if (Str1.equals("2")) Ln1Foul = 2;       //And set Lane1 foul status to 2 if 2'nd lane to foul
      Str1 = Time_String.substring(9, 10);     //Extract Breakout status for Lane1
      if (Str1.equals("1")) Brkout1flag = 1;   //And set the Lane1 Breakout flag accordingly
        else Brkout1flag = 0;
      Str1 = Time_String.substring(11, 19);    //Extract Lane1 Reaction time (aaaa.aaa)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      if (Ln1Foul>0) Value = Value * -1.0;     //Make it negative if foul is set
      ReactTime[0] = Truncate(Value);          //and trim to 3 decimal places
      Str1 = Time_String.substring(20, 28);    //Extract Lane1 60 Ft time (bbbb.bbb)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Trk60ftTime[0] = Truncate(Value);        //and trim to 3 decimal places
      Str1 = Time_String.substring(29, 37);    //Extract Lane1 660 Ft time (cccc.ccc)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Trk660ftTime[0] = Truncate(Value);       //and trim to 3 decimal places
      Str1 = Time_String.substring(38, 46);    //Extract Lane1 Finish time (dddd.ddd)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Trk1320ftTime[0] = Truncate(Value);      //and trim to 3 decimal places
      Str1 = Time_String.substring(47, 55);    //Extract Lane1 1320 Ft speed (eeee.eee)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Speed[0] = Truncate(Value);              //and trim to 3 decimal places
      Str1 = Time_String.substring(56, 57);    //Extract Lane1 win status (z)
      if (Str1.equals("1")) Win1flag = 1;      //And set the Lane1 Win flag accordingly
        else Win1flag = 0;
      inString = "";                           //Clear Race times string once processed
    }
    if (inString.indexOf("Lane2") >= 0) {      //Check for receipt of Lane2 Race times
      Time_String = inString;                  //and save it to a temp holding variable
      //Format: Lane2: x y aaaa.aaa bbbb.bbb cccc.ccc dddd.ddd eeee.eee z
      //Where: x=Foul flag, y=Breakout flag, aaaa.aaa=RT, bbbb.bbb=60Ft ET, cccc.ccc=660Ft ET, 
      //       dddd.ddd==1320Ft ET, eeee.eee=1320Ft Speed, z=Win flag
      Str1 = Time_String.substring(7, 8);      //Extract foul status for Lane2 (x)
      Ln2Foul = 0;                             //Preset Lane2 foul status to not fouled
      if (Str1.equals("1")) Ln2Foul = 1;       //And set Lane2 foul status to 1 if 1'st lane to foul
      if (Str1.equals("2")) Ln2Foul = 2;       //And set Lane2 foul status to 2 if 2'nd lane to foul
      Str1 = Time_String.substring(9, 10);     //Extract Breakout status for Lane2
      if (Str1.equals("1")) Brkout2flag = 1;   //And set the Lane2 Breakout flag accordingly
        else Brkout2flag = 0;
      Str1 = Time_String.substring(11, 19);    //Extract Lane2 Reaction time (aaaa.aaa)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      if (Ln2Foul>0) Value = Value * -1.0;     //Make it negative if foul is set
      ReactTime[1] = Truncate(Value);          //and trim to 3 decimal places
      Str1 = Time_String.substring(20, 28);    //Extract Lane2 60 Ft time (bbbb.bbb)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Trk60ftTime[1] = Truncate(Value);        //and trim to 3 decimal places
      Str1 = Time_String.substring(29, 37);    //Extract Lane2 660 Ft time (cccc.ccc)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Trk660ftTime[1] = Truncate(Value);       //and trim to 3 decimal places
      Str1 = Time_String.substring(38, 46);    //Extract Lane2 Finish time (dddd.ddd)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Trk1320ftTime[1] = Truncate(Value);      //and trim to 3 decimal places
      Str1 = Time_String.substring(47, 55);    //Extract Lane2 1320 Ft speed (eeee.eee)
      Value = float (Str1) - 1000.0;           //Convert it to a floating point number
      Speed[1] = Truncate(Value);              //and trim to 3 decimal places
      Str1 = Time_String.substring(56, 57);    //Extract Lane2 win status (z)
      if (Str1.equals("1")) Win2flag = 1;      //And set the Lane2 Win flag accordingly
        else Win2flag = 0;
      inString = "";                           //Clear Race times string once processed
    }
    if (inString.indexOf("FIN") >= 0) {        //Check for receipt of 'Finished' state
      Race_State = 5;                          //Set the race state to 5 (5=finished)
      Set_Race_State();                        //and call subroutine to display the race state
      Get_datetime();                          //Get & set the race date & time stamp values
      if (Race_Number == 0) Disp_Message(4);   //Call subroutine to display alternate message if no races ran
      else Disp_Message(3);                    //or display default display message
      NextRaceflag = true;                     //Enable Next Race button
      Draw_NextRace_Button();                  //Call subroutine to update Next Race button display
      inString = "";                           //Clear 'Finished' message once processed
      //Calculate Overall Times & Margin
      LaneET[0] = Trk1320ftTime[0];            //Lane1 ET
      LaneET[1] = Trk1320ftTime[1];            //Lane2 ET
      if(ReactTime[0]<900.0 && LaneET[0]<900.0) { //Check if Lane1 times are valid
        if (Ln1Foul==0) {                      //If Reaction Time is positive
          Value = ReactTime[0] + LaneET[0];    //calculate overall race time
          OverallTime[0] = Truncate(Value);    //and trim to 3 decimal places
        }
        else OverallTime[0] = LaneET[0];       //OverallTime=ElapsedTime if stage sensor tripped before green light
      }
      else OverallTime[0] = 999.999;           //Set overall time to default value if invalid lane times
      if(ReactTime[1]<900.0 && LaneET[1]<900.0) { //Check if Lane2 times are valid
        if (Ln2Foul==0) {                      //If Reaction Time is positive
          Value = ReactTime[1] + LaneET[1];    //calculate overall race time
          OverallTime[1] = Truncate(Value);    //and trim to 3 decimal places
        }
        else OverallTime[1] = LaneET[1];       //OverallTime=ElapsedTime if stage sensor tripped before green light
      }
      else OverallTime[1] = 999.999;           //Set overall time to default value if invalid lane time
      if (OverallTime[0]<900.0 && OverallTime[1]<900.0) { //Do only if both lanes completed race
        if (Win1flag==1) {                     //Do if Lane1 is winner
          Margin[0] = OverallTime[1] - OverallTime[0]; //Calc. Lane1 margin
          Margin[1] = 999.999;                 //Set lane2 margin to invalid so it doesn't display
        }
        if (Win2flag==1) {                     //Do if Lane2 is winner
          Margin[1] = OverallTime[0] - OverallTime[1]; //Calc. Lane2 margin
          Margin[0] = 999.999;                 //Set lane1 margin to invalid so it doesn't display
        }
      }
      else {
        Margin[0] = 999.999;
        Margin[1] = 999.999;
      }
      Set_Racer_Status();                      //Call subroutine to get racer's status
      Display_Results();                       //Call subroutine to display the results
      if (Race_Number == 0) PracticeRaceflag = true; //Set flag if results are from a practice race
        else PracticeRaceflag = false;
    
      //Write results to the Resultstable if not a practice race
      if (Race_Number > Races_Ran) {           //Do only if a new race
        Races_Ran = Race_Number;               //Bump the Races Ran count
        Resultstable.addRow();                 //Add new row to the table
      }
      if (Race_Number > 0) {                   //Don't save if practice run (i.e. Race Number = #)
        Tablerow = Race_Number - 1;                                     //Set the table row index
        Resultstable.setInt(Tablerow, "Race", Race_Number);             //Save Race Number
        Resultstable.setString(Tablerow, "L1_Car", Carnum[0]);          //Save Lane1 Car#
        Resultstable.setString(Tablerow, "L1_Class", RClass[0]);        //Save Lane1 race class
        Resultstable.setInt(Tablerow, "L1_Foul", Ln1Foul);              //Save Lane1 foul status
        Resultstable.setInt(Tablerow, "L1_BrkOutflag", Brkout1flag);    //Save Lane1 Breakout status
        Resultstable.setFloat(Tablerow, "L1_Dialin", DialinTime[0]);    //Save Lane1 dial-in time
        Resultstable.setFloat(Tablerow, "L1_React", ReactTime[0]);      //Save Lane1 reaction time
        Resultstable.setFloat(Tablerow, "L1_60Ft", Trk60ftTime[0]);     //Save Lane1 60 Ft ET
        Resultstable.setFloat(Tablerow, "L1_660Ft", Trk660ftTime[0]);   //Save Lane1 660 Ft ET
        Resultstable.setFloat(Tablerow, "L1_1320Ft", Trk1320ftTime[0]); //Save Lane1 1320 Ft ET
        Resultstable.setFloat(Tablerow, "L1_Speed", Speed[0]);          //Save Lane1 1320 Ft speed
        Resultstable.setFloat(Tablerow, "L1_Ovl", OverallTime[0]);      //Save Lane1 overall time
        Resultstable.setFloat(Tablerow, "L1_Margin", Margin[0]);        //Save Lane1 Margin
        Resultstable.setFloat(Tablerow, "L1_Winflag", Win1flag);        //Save Lane1 win status
        Resultstable.setString(Tablerow, "L2_Car", Carnum[1]);          //Save Lane2 Car#
        Resultstable.setString(Tablerow, "L2_Class", RClass[1]);        //Save Lane2 race class
        Resultstable.setInt(Tablerow, "L2_Foul", Ln2Foul);              //Save Lane2 foul status
        Resultstable.setInt(Tablerow, "L2_BrkOutflag", Brkout2flag);    //Save Lane2 breakout status
        Resultstable.setFloat(Tablerow, "L2_Dialin", DialinTime[1]);    //Save Lane2 dial-in time
        Resultstable.setFloat(Tablerow, "L2_React", ReactTime[1]);      //Save Lane2 reaction time
        Resultstable.setFloat(Tablerow, "L2_60Ft", Trk60ftTime[1]);     //Save Lane2 60 Ft ET
        Resultstable.setFloat(Tablerow, "L2_660Ft", Trk660ftTime[1]);   //Save Lane2 660 Ft ET
        Resultstable.setFloat(Tablerow, "L2_1320Ft", Trk1320ftTime[1]); //Save Lane2 1320 Ft ET
        Resultstable.setFloat(Tablerow, "L2_Speed", Speed[1]);          //Save Lane2 1320 Ft speed
        Resultstable.setFloat(Tablerow, "L2_Ovl", OverallTime[1]);      //Save Lane2 overall time
        Resultstable.setFloat(Tablerow, "L2_Margin", Margin[1]);        //Save Lane2 Margin
        Resultstable.setFloat(Tablerow, "L2_Winflag", Win2flag);        //Save Lane2 win status
        Resultstable.setString(Tablerow, "Date", TS_date);              //Save time slip date stamp
        Resultstable.setString(Tablerow, "Time", TS_time);              //Save time slip time stamp
        //Save all results to the datafile each time new results are available. Note that by default
        //the file is saved to the same folder that contains this sketch.
        saveTable(Resultstable, Datafile);
      }
      if (IndyTSflag && Lane_Usage==3) {
        Create_Timeslip(1);                     //Call subroutine to create Lane1 time slip file
        if (Autoprintcopies > 0) {              //Print Lane1 time slips?
          for (i=0; i<Autoprintcopies; i++) {   //Loop for number of copies
            launch(Prt_Cmd);                    //Launch Windows command to  print time slip
            delay(1000);                        //Wait till file is printed before creating next timeslip
          }
        }
        Create_Timeslip(2);                     //Call subroutine to create Lane2 time slip file
        if (Autoprintcopies > 0) {              //Print Lane2 time slips?
          for (i=0; i<Autoprintcopies; i++) {   //Loop for number of copies
            launch(Prt_Cmd);                    //Launch Windows command to  print time slip
            delay(100);                         //Wait till file is printed
          }
        }
      }
      else {
        Create_Timeslip(Lane_Usage);            //Call subroutine to create time slip file
        if (Autoprintcopies > 0) {              //Print Time Slips?
          for (i=0; i<Autoprintcopies; i++) {   //Loop for number of copies
            launch(Prt_Cmd);                    //Launch Windows command to  print time slip
            delay(100);                         //Wait till file is printed
          }
        }
      }
      inString = "";                           //Clear Race times string once processed
    }
  }
} //End of "draw" loop

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                            SUBROUTINES                                               *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void serialEvent(Serial p) {                         //Serial data handling routine
  inString = p.readString();
}

void mouseClicked() {                                //Mouse Click handling routine
  //This routine is called when the mouse button is clicked. Mouse X & Y coordinates are tested to see
  //if the mouse pointer is over a button and will respond accordingly if so.
  x = mouseX;                                        //Get mouse X-coordinate
  y = mouseY;                                        //Get mouse Y-coordinate
  if (SetupPgSelect) { //DO IF SETUP PAGE SELECTED
    //Test for Select Comm button click
    if ((x>180) && (x<300) && (y>180) && (y<240)) {  //Select Comm button clicked?
      if (!ComPort_Avail) Select_Comm();             //Call subroutine to select com port
      if (ComPort_Avail) {                           //Do if COM port available
        inString = "";                               //Clear input string
        myPort.write("C");                           //Send the letter C for comm check request
        myPort.write(lf);                            //followed by a line-feed
        fill(200);                                   //Set color to setup screen background
        noStroke();                                  //Disable object draw outline
        rect(315,180,400,60);                        //Clear any previous com check messages
        stroke(0);                                   //Enable object draw outline
        i = millis() + 2000;                         //Get CPU time + 2 seconds
        while (millis() < i) {                       //Loop for max of 2 seconds
          if (inString.indexOf("@") >= 0) {          //Did timer send acknowledge?
            textSize(25);                            //Set text size to 25 pixels
            textAlign(LEFT);                         //Set text alignment to center
            fill(0);                                 //Set text color to black
            text("Com Check OK on Port: " + Portname, 320, 220); //Draw text
            ComSelectedflag = true;                  //Set Com Select flag
            inString = "";                           //Clear input string
            break;                                   //exit the while loop
          }
        }
      }
    }
    //Test for Display Title change request
    if ((x>320) && (x<980) && (y>245) && (y<275)) {
      prompt = "Enter Display Title";                //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for display title
      if (Str1 != null) {                            //Check if entry got canceled
        Disp_Title = Str1;                           //Update display title
        Draw_User_Parameters();                      //Update user parameters screen
      }
    }
    //Test for Time Slip Header Line1 change request
    if ((x>320) && (x<720) && (y>295) && (y<325)) {
      prompt = "Enter Time Slip Text\nFor Header Line1\n(32 Characters Max) "; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for TS Header Line1
      if (Str1 != null) {                            //Check if entry got canceled
        if (Str1.length()>32) Str1 = Str1.substring(0,32); //Trim length if > 32 characters
        TS_Header1 = Str1;                           //Update TS Header Line1
        Draw_User_Parameters();                      //Refresh user parameters screen
      }
    }
    //Test for Time Slip Header Line2 change request
    if ((x>320) && (x<720) && (y>345) && (y<375)) {
      prompt = "Enter Time Slip Text\nFor Header Line2\n(32 Characters Max) "; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for TS Header Line2
      if (Str1 != null) {                            //Check if entry got canceled
        if (Str1.length()>32) Str1 = Str1.substring(0,32); //Trim length if > 32 characters
        TS_Header2 = Str1;                           //Update TS Header Line2
        Draw_User_Parameters();                      //Refresh user parameters screen
      }
    }
    //Test for Time Slip Footer Line1 change request
    if ((x>320) && (x<720) && (y>395) && (y<425)) {
      prompt = "Enter Time Slip Text\nFor Footer Line1\n(32 Characters Max) "; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for TS Footer Line1
      if (Str1 != null) {                            //Check if entry got canceled
        if (Str1.length()>32) Str1 = Str1.substring(0,32); //Trim length if > 32 characters
        TS_Footer1 = Str1;                           //Update TS Footer Line1
        Draw_User_Parameters();                      //Refresh user parameters screen
      }
    }
    //Test for Time Slip Footer Line2 change request
    if ((x>320) && (x<720) && (y>445) && (y<475)) {
      prompt = "Enter Time Slip Text\nFor Footer Line2\n(32 Characters Max) "; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for TS Footer Line2
      if (Str1 != null) {                            //Check if entry got canceled
        if (Str1.length()>32) Str1 = Str1.substring(0,32); //Trim length if > 32 characters
        TS_Footer2 = Str1;                           //Update TS Footer Line2
        Draw_User_Parameters();                      //Refresh user parameters screen
      }
    }
    //Test for Time Slip Number of Copies request
    if ((x>320) && (x<370) && (y>495) && (y<525)) {
      prompt = "Enter Number of Time Slip Copies\nto Auto-Print  (Range: 0 - 9)"; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for number of Auto-print copies
      if (Str1 != null) {                            //Check if entry got canceled
        if (Str1.length()>1) Str1 = Str1.substring(0,1); //Trim length if > 1 characters
        i = int(Str1);                               //Convert to integer
        if (i>=0 && i<10) {                          //Limit number of copies to 9
          Autoprintcopies = i;                       //Update Auto-print copies value
          Draw_User_Parameters();                    //Refresh user parameters screen
        }
      }
    }
    //Test for Timeout duration request
    if ((x>320) && (x<370) && (y>545) && (y<575)) {
      prompt = "Enter the Timeout Duration\nin Seconds  (Range: 5 - 60)"; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for timeout duration
      if (Str1 != null) {                            //Check if entry got canceled
        if (Str1.length()>2) Str1 = Str1.substring(0,2); //Trim length if > 2 characters
        i = int(Str1);                               //Convert to integer
        if (i>4 && i<61) {                           //Limit range to 5 - 60 seconds
          timeout = i;                               //Update timeout duration value
          Draw_User_Parameters();                    //Refresh user parameters screen
        }
      }
    }
    //Test for Metric flag set/reset request
    if ((x>320) && (x<370) && (y>595) && (y<625)) {
      MetricFlag = !MetricFlag;                      //Toggle metric flag
      Draw_User_Parameters();                        //Refresh user parameters screen
    }
    //Test for Proceed to Race Page button
    if ((x>520) && (x<680) && (y>610) && (y<670)) {
      if (ComSelectedflag) {                         //Has COM port been selected
        SetupPgSelect = false;                       //Clear flag to exit setup page
        //Update Parameter table
        Parmtable.setString(0, "VALUE", Disp_Title); //Save display title
        Parmtable.setString(1, "VALUE", TS_Header1); //Save TS header Line1 text
        Parmtable.setString(2, "VALUE", TS_Header2); //Save TS header Line2 text
        Parmtable.setString(3, "VALUE", TS_Footer1); //Save TS footer Line1 text
        Parmtable.setString(4, "VALUE", TS_Footer2); //Save TS footer Line2 text
        Parmtable.setInt(5, "VALUE", Autoprintcopies); //Save Auto-print copies value
        Parmtable.setInt(6, "VALUE", timeout);       //Save timeout value
        if (MetricFlag) Parmtable.setString(7, "VALUE", "T"); //Save Metric flag state
         else Parmtable.setString(7, "VALUE", "F");  //Save Metric flag state
        saveTable(Parmtable, parmFilename);          //Save table...Saved in same folder as this sketch
        //Send Initial setup to timer
        Str1 = "TO:" + nf(timeout, 2);               //Convert to 5 character string
        myPort.write(Str1);                          //Send timeout string to timer
        myPort.write(lf);                            //followed by a line-feed
        delay(10);                                   //Wait a while
        myPort.write("M:0");                         //Send default tree mode to timer
        myPort.write(lf);                            //followed by a line-feed
        delay(10);                                   //Wait a while
        myPort.write("AS:0");                        //Send default Auto Start mode to timer
        myPort.write(lf);                            //followed by a line-feed
        delay(10);                                   //Wait a while
        myPort.write("LN:3");                        //Send default Lane Usage mode to timer
        myPort.write(lf);                            //followed by a line-feed
        delay(10);                                   //Wait a while
        myPort.write("TBL:0");                       //Send default Top Bulb Lock mode to timer
        myPort.write(lf);                            //followed by a line-feed
        delay(10);                                   //Wait a while
        myPort.write("DBR:0");                       //Send default Double Red mode to timer
        myPort.write(lf);                            //followed by a line-feed
        delay(10);                                   //Wait a while
        if (MetricFlag) myPort.write("KMH:1");       //Send Metric Speed selection to Arduino
        else myPort.write("KMH:0");                  //Send Metric Speed selection to Arduino
        myPort.write(lf);                            //followed by a line-feed
        Init_Race_Page();                            //Call subroutine to initialize the race page
      }
      else {
        prompt = "Must Select COM Port\nbefore proceeding";
        showMessageDialog(null,prompt);              //Display Dialog
      }
    }
  }
  else { //DO IF RACE PAGE ACTIVE
    //Test for RESET button and send the reset command to the Arduino if so.
    if ((x>190) && (x<230) && (y>570) && (y<610)) {
      myPort.write("RS");                            //Send Reset command to timer
      myPort.write(lf);                              //followed by a line-feed
      NextRaceflag = true;                           //Enable Next Race button (in case it was disabled)
      Draw_NextRace_Button();                        //And update display
    }
    //Test for Next Race select button and perform necessary actions
    if ((x>255) && (x<335) && (y>535) && (y<595)) {
      if (NextRaceflag && (Races_Ran<=Max_Races)) {  //Only allow if next race flag is TRUE & races ran <= 999
        if (Holdflag) {                              //Check if Holdflag already cleared
          Holdflag = false;                          //Reset the Hold flag
          myPort.write("C");                         //Send 'Continue' command to timer
          myPort.write(lf);                          //followed by a line-feed
        }
        Race_Number = Races_Ran + 1;                 //Set the current race number
        Draw_Racebox();                              //Redisplay new race value
        myPort.write("RS");                          //Send Reset command to timer
        myPort.write(lf);                            //followed by a line-fee
      }
    }
    //Test for Race Start select button and perform necessary actions
    if ((x>255) && (x<335) && (y>600) && (y<660)) {
      if (Race_State==2 && Race_Number>Races_Ran) {  //Do only if in 'Staged' state & Race# set for next race
        myPort.write("ST");                          //Send the start command to Arduino
        myPort.write(lf);                            //followed by a line-feed
      }
      else {
        prompt = "TIMER NOT READY";                  //Build the prompt string
        showMessageDialog(null,prompt);              //Display Dialog
      }
    }
    //Test for Print Time Slip button and do if so.
    if ((x>380) && (x<420) && (y>570) && (y<610)) {
      if (PracticeRaceflag || Race_Number<=Races_Ran) { //Only do if valid data is selected
        if (IndyTSflag && Lane_Usage==3) {
          Create_Timeslip(1);                        //Call subroutine to create Lane1 time slip file
          launch(Prt_Cmd);                           //Launch Windows command to  print time slip
          delay(1000);                               //Wait till file is printed before creating next timeslip
          Create_Timeslip(2);                        //Call subroutine to create Lane2 time slip file
          launch(Prt_Cmd);                           //Launch Windows command to  print time slip
        }
        else {
          Create_Timeslip(Lane_Usage);               //Call subroutine to create time slip file
          launch(Prt_Cmd);                           //Launch Windows command to  print time slip
        }
        prompt = "Timeslip Printed";                 //Tell operator the time slip has printed
      }
      else prompt = "Must Select Race\nThat Has Been Run";
      showMessageDialog(null,prompt);                //Display Dialog
    }
    //Test for Up Arrow button and increment Race Number value by one if so.
    if ((x>560) && (x<590) && (y>550) && (y<580)) {
      if (Race_Number < Races_Ran) {
        Race_Number++;                               //Increment Race number if < number of races ran
        Draw_Racebox();                              //and redisplay new value
        Index1 = Race_Number - 1;                    //Set table index pointer
        Redisplay_RaceResults();                     //Call subroutine to redisplay results
      }
      else {
        if (Race_Number == Races_Ran && Race_State == 1) {
          Race_Number++;                             //Increment Race number if ready to run the next race
          Draw_Racebox();                            //and redisplay new value
          Clear_Data();                              //Clear displayed data
          Clear_Msgbox();                            //Clear Message box
          if (Holdflag) {                            //Check if Holdflag already cleared
            Holdflag = false;                        //Reset the Hold flag
            myPort.write("C");                       //Send 'Continue' command to timer
            myPort.write(lf);                        //followed by a line-feed
          }
        }
      }
    }
    //Test for Down Arrow button and decrement Race Number value by one if so.
    if ((x>560) && (x<590) && (y>600) && (y<630)) {
      if (Race_Number > 1) {
        Race_Number--;                               //Decrement Race number if < 1
        Draw_Racebox();                              //and redisplay new value
        Index1 = Race_Number - 1;                    //Set table index pointer
        Redisplay_RaceResults();                     //Call subroutine to redisplay results
        if (!Holdflag) {                             //Check if Holdflag already set
          Holdflag = true;                           //Set the Hold flag
          myPort.write("H");                         //Send 'Hold' command to timer
          myPort.write(lf);                          //followed by a line-feed
        }
      }
    }
    //Test for Load Roster button
    if ((x>650) && (x<765) && (y>355) && (y<380)) {
      if (!RosterLoadedflag) {                       //Check if roster is loaded
        Load_Roster();                               //Call load roster subroutine
        RosterLoadedflag = true;                     //Set flag to indicate roster is loaded
      }
      else {
        prompt = "Roster already loaded.";
        showMessageDialog(null,prompt);              //Display the prompt for acknowledgement
      }
    }
    //Test for Add Driver button
    if ((x>785) && (x<900) && (y>355) && (y<380)) {
      n = Rostertable.getRowCount();                 //Get number of roster entries
      AddDriverflag = false;                         //Preset flag
      Rostertable.addRow();                          //Add new row to the table
      prompt = "Enter Driver Name\n(32 Characters Max) "; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for driver name
      if (Str1!=null && Str1.length()>0) {           //Check if entry got canceled
        if (Str1.length()>32) Str1 = Str1.substring(0,32); //Trim length if > 32 characters
        Rostertable.setString(n,"Name",Str1);        //Add driver name to the roster table
        AddDriverflag = true;                        //Set flag to signify driver added
      }
      else Rostertable.setString(n,"Name","No Name");//Add default name to the roster table
      prompt = "Enter Driver's Car Number\n(6 Characters Max) "; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for driver name
      if (Str1!=null && Str1.length()>0) {           //Check if entry got canceled
        if (Str1.length()>6) Str1 = Str1.substring(0,6); //Trim length if > 6 characters
        Rostertable.setString(n,"CarNo",Str1);       //Add driver car# to the roster table
        AddDriverflag = true;                        //Set flag to signify driver added
      }
      else Rostertable.setString(n,"CarNo","---");    //Add default car# to the roster table
      prompt = "Enter Driver's Race Class\n(10 Characters Max) "; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for driver name
      if (Str1!=null && Str1.length()>0) {           //Check if entry got canceled
        if (Str1.length()>10) Str1 = Str1.substring(0,10); //Trim length if > 10 characters
        Rostertable.setString(n,"Class",Str1);       //Add driver class to the roster table
        AddDriverflag = true;                        //Set flag to signify driver added
      }
      else Rostertable.setString(n,"Class","----");   //Add default class to the roster table
      prompt = "Enter Driver's Dial-In time in seconds\nEnter 0 if none\nFormat: XX.YY"; //Set prompt
      Str1 = showInputDialog(prompt);                //Prompt user for dial-in time
      if (Str1!=null && Str1.length()>0) {           //Check if entry got canceled
        Value = float(Str1);                         //Convert to floating point value
        if (Value<0.0 || Value>20.0) {               //Check if valid value
          prompt = "Invalid dial-in time entered. Setting to 0.0";
          showMessageDialog(null,prompt);            //Display message
          Value = 0.0;
        }
        Rostertable.setFloat(n,"Dialin",Value);      //Add driver dialin to the roster table
        AddDriverflag = true;                        //Set flag to signify driver added
      }
      else Rostertable.setFloat(n,"Dialin",0.0);     //Add default dialin to the roster table
      if (!AddDriverflag) Rostertable.removeRow(n);  //Delete added row if add driver was canceled
      RosterIndex = n-8;                             //Set roster pointer
      if (RosterIndex<0) RosterIndex=0;              //Set roster pointer
      Display_Roster();                              //Update roster display
      RosterLoadedflag = true;                       //Set to signify there is at least on name in the roster
    }
    //Test for Delete Driver button
    if ((x>920) && (x<1035) && (y>355) && (y<380)) {
      n = Rostertable.getRowCount();                 //Get number of roster entries
      if (n > 0) {                                   //Do only if roster has entries
        prompt = "Enter Roster Entry Number of\ndriver you want to delete"; //Set prompt
        Str1 = showInputDialog(prompt);              //Prompt user for entry number
        if (Str1!=null) {
          if (int(Str1)>0 && int(Str1)<=n) {         //Check entry validity
            j = int(Str1);                           //Convert entry to integer value
            Rostertable.removeRow(j-1);              //Remove the selected driver from table
            RosterIndex=0;                           //Set roster pointer
            Display_Roster();                        //Update roster display
          }
          else {                                     //Do if entry is invalid
            prompt = "INVALID ENTRY";
            showMessageDialog(null,prompt);          //Display message
          }
        }
      }
      else {
        prompt = "No Drivers in roster\nAdd a driver or load a roster file";
        showMessageDialog(null,prompt);              //Display message
      }
    }
    //Test for Save Roster button
    if ((x>1055) && (x<1170) && (y>355) && (y<380)) {
      saveTable(Rostertable, RosterFilename);
      prompt = "Roster Saved\nTo: " + RosterFilename;
      showMessageDialog(null,prompt);                //Display message
    }
    //Test for Roster Down-Arrow button
    if ((x>1140) && (x<1170) && (y>320) && (y<350)) {
      if (Rostertable.getRowCount() > RosterIndex+8) { //Check roster index for max value
        RosterIndex++;                               //Increment roster index
        Display_Roster();                            //Update roster display
      }
    }
    //Test for Roster Up-Arrow button
    if ((x>1140) && (x<1170) && (y>100) && (y<130)) {
      if (RosterIndex > 0) {                         //Check roster index for max value
        RosterIndex--;                               //Deccrement roster index
        Display_Roster();                            //Update roster display
      }
    }
    //Test for driver select from roster
    if ((x>650) && (x<700) && (y>100) && (y<350)) {
      if (Race_State==1) {                           //Allow only if timer in ready state
        n = Rostertable.getRowCount();
        if (n > 0) {                                 //Check if roster has entries
          if ((y>100) && (y<125)) j = RosterIndex;   //Do if 1'st row is selected
          if ((y>125) && (y<150)) j = RosterIndex+1; //Do if 2'nd row is selected
          if ((y>150) && (y<175)) j = RosterIndex+2; //Do if 3'd row is selected
          if ((y>175) && (y<200)) j = RosterIndex+3; //Do if 4'th row is selected
          if ((y>200) && (y<225)) j = RosterIndex+4; //Do if 5'th row is selected
          if ((y>225) && (y<250)) j = RosterIndex+5; //Do if 6'th row is selected
          if ((y>250) && (y<275)) j = RosterIndex+6; //Do if 7'th row is selected
          if ((y>275) && (y<300)) j = RosterIndex+7; //Do if 8'th row is selected
          if ((y>300) && (y<325)) j = RosterIndex+8; //Do if 9'th row is selected
          if ((y>325) && (y<350)) j = RosterIndex+9; //Do if 10'th row is selected
          if (j < n) {                               //Do only if selected row has a driver
            myPort.write("H");                       //Send 'Hold' command to timer
            myPort.write(lf);                        //followed by a line-feed
            textSize(30);                            //Set text size to 30 pixels
            textAlign(CENTER);                       //Set text justification to center
            Str1 = "LANE ASSIGN SELECT";             //Set prompt header
            prompt = "Assign driver# " + (j+1) + " to lane;"; //Set prompt
            Object[] options = {"1", "2", "Cancel"};
            int input = showOptionDialog(null, prompt, Str1, DEFAULT_OPTION, INFORMATION_MESSAGE,null, options, null);
            if (input == 0) {                        //Is Lane1 selected?
              RClass[0] = Rostertable.getString(j,"Class"); //Get driver class from roster
              Carnum[0] = Rostertable.getString(j,"CarNo"); //Get driver car# from roster
              if (HandiCapEnblflag) {                //Do if handicap enabled
                DialinTime[0] = Rostertable.getFloat(j,"Dialin"); //Get driver dial-in time from roster
                n = int(DialinTime[0]*1000.0);        //Convert dialin to miliseconds
              }
              else {                                 //Do if handicap disabled
                DialinTime[0] = 0.0;                 //Set dial-in to 0
                n = 0;                               //Set dial-in to 0
              }
              Str1 = "D1:" + nf(n,5);                //Format dialin string
              myPort.write(Str1);                    //And send to timer
              myPort.write(lf);                      //Followed by a line-feed
              fill(100);                             //Set rectangle color
              rect(90,60,140,30,10);                 //Clear Lane1 Racer's Class
              rect(90,100,140,30,10);                //Clear Lane1 Racer's Car#
              rect(90,140,140,30,10);                //Clear Lane1 Dial-In time
              fill(255);                             //Set text color to white
              text(RClass[0],160,85);                //Display Class
              text(Carnum[0],160,125);               //Display Car#
              Str1 = nf(DialinTime[0],0,2);          //Convert to formatted string
              text(Str1,160,165);                    //Display Dial-in
            }
            if (input == 1) {                        //Is Lane2 selected?
              RClass[1] = Rostertable.getString(j,"Class"); //Get driver class from roster
              Carnum[1] = Rostertable.getString(j,"CarNo"); //Get driver car# from roster
              if (HandiCapEnblflag) {                //Do if handicap enabled
                DialinTime[1] = Rostertable.getFloat(j,"Dialin"); //Get driver dial-in time from roster
                n = int(DialinTime[1]*1000.0);        //Convert dialin to miliseconds
              }
              else {                                 //Do if handicap disabled
                DialinTime[0] = 0.0;                 //Set dial-in to 0
                n = 0;                               //Set dial-in to 0
              }
              Str1 = "D2:" + nf(n,5);                //Format dialin string
              if (!HandiCapEnblflag) Str1 = "D2:00000"; //Clear Lane2 dial-in to timer
              myPort.write(Str1);                    //And send to timer
              myPort.write(lf);                      //Followed by a line-feed
              fill(100);                             //Set rectangle color
              rect(350,60,140,30,10);                //Clear Lane2 Racer's Class
              rect(350,100,140,30,10);               //Clear Lane2 Racer's Car#
              rect(350,140,140,30,10);               //Clear Lane2 Dial-In time
              fill(255);                             //Set text color to white
              text(RClass[1],420,85);                //Display Class
              text(Carnum[1],420,125);               //Display Car#
              Str1 = nf(DialinTime[1],0,2);          //Convert to formatted string
              text(Str1,420,165);                    //Display Dial-in
            }
            myPort.write("C");                       //Send 'Continue' command to timer
            myPort.write(lf);                        //followed by a line-feed
          }
        }
        else {
          prompt = "No Drivers in roster\nAdd a driver or load a roster file";
          showMessageDialog(null,prompt);            //Display message
        }
      }
      else {
        prompt = "Sorry...Can only add driver\nwhile timer is in Ready state";
        showMessageDialog(null,prompt);              //Display message
      }
    }
    //Test for tree mode select
    if ((x>780) && (x<855) && (y>530) && (y<555)) {
      if (Race_State<2 || Race_State==5) {         //Allow only if not in a staged or racing state
        myPort.write("H");                         //Send 'Hold' command to timer
        myPort.write(lf);                          //followed by a line-feed
        Str1 = "XMAS TREE MODE SELECT";            //Set prompt header
        prompt = "Select XMas Tree Mode";          //Set prompt
        Object[] options = {"Std", "Pro", "Outlaw", "Cancel"};
        int input = showOptionDialog(null, prompt, Str1, DEFAULT_OPTION, INFORMATION_MESSAGE,null, options, null);
        if (input == 0) Tree_Mode = "M:0";         //Set 'Standard' Xmas Tree mode
        if (input == 1) Tree_Mode = "M:1";         //Set 'Pro' Xmas Tree mode
        if (input == 2) Tree_Mode = "M:2";         //Set 'Outlaw' Xmas Tree mode
        myPort.write(Tree_Mode);                   //Send Xmas Tree mode to Arduino
        myPort.write(lf);                          //followed by a line-feed
        delay(10);                                 //Wait for Arduino timer to process
        myPort.write("C");                         //Send 'Continue' command to timer
        myPort.write(lf);                          //followed by a line-feed
        Draw_RaceOptions();                        //Call subroutine to update race option display
      }
      else {
        prompt = "Must be in Ready or Finished state\nto select Xmas Tree mode";
        showMessageDialog(null,prompt);            //Display message
      }
    }
    //Test for Auto Start option select
    if ((x>780) && (x<855) && (y>560) && (y<585)) {
      if (Race_State<2 || Race_State==5) {         //Allow only if not in a staged or racing state
        Autostartflag = !Autostartflag;            //Toggle flag
        if (Autostartflag) {                       //Is Auto start currently selected
          myPort.write("AS:1");                    //Send timer Auto-Start command to Arduino
          myPort.write(lf);                        //followed by a line-feed
        }
        else {
          myPort.write("AS:0");                    //Send timer Manual-Start command to Arduino
          myPort.write(lf);                        //followed by a line-feed
        }
        Draw_RaceOptions();                        //Call subroutine to update race option display
      }
      else {
        prompt = "Must be in Ready or Finished state\nto select Auto/Manual Start mode";
        showMessageDialog(null,prompt);            //Display message
      }
    }
    //Test for Handicap Enable/Disable select
    if ((x>780) && (x<855) && (y>590) && (y<615)) {
      if (Race_State<2 || Race_State==5) {         //Allow only if not in a staged or racing state
        HandiCapEnblflag = !HandiCapEnblflag;      //Toggle flag
        Draw_RaceOptions();                        //Call subroutine to update race option display
      }
      else {
        prompt = "Must be in Ready or Finished state\nto change Handicap Enbl/Dsbl option";
        showMessageDialog(null,prompt);            //Display message
      }
    }
    //Test for Single/Dual Timeslip select
    if ((x>780) && (x<880) && (y>620) && (y<645)) {
      if (Race_State<2 || Race_State==5) {         //Allow only if not in a staged or racing state
        IndyTSflag = !IndyTSflag;                  //Toggle flag
        Draw_RaceOptions();                        //Call subroutine to update race option display
      }
      else {
        prompt = "Must be in Ready or Finished state\nto select Timeslip format";
        showMessageDialog(null,prompt);            //Display message
      }
    }
    //Test for Active Lane select
    if ((x>1050) && (x<1125) && (y>530) && (y<555)) {
      if (Race_State<2 || Race_State==5) {         //Allow only if not in a staged or racing state
        myPort.write("H");                         //Send 'Hold' command to timer
        myPort.write(lf);                          //followed by a line-feed
        Str1 = "LANE USAGE SELECT";                //Set prompt header
        prompt = "Which Lanes Will Be Racing? ";   //Set prompt
        Object[] options = {"Lane 1", "Lane 2", "Lanes 1 & 2", "Cancel"};
        int input = showOptionDialog(null, prompt, Str1, DEFAULT_OPTION, INFORMATION_MESSAGE,null, options, null);
        if (input == 0) {                          //Is Lane1 selected?
          Lane_Usage = 1;                          //Set lane usage to Lane1
          Str1 = "LN:1";                           //Set lane usage command for Lane1
        }
        if (input == 1) {                          //Is Lane2 selected?
          Lane_Usage = 2;                          //Set lane usage to Lane2
          Str1 = "LN:2";                           //Set lane usage command for Lane2
        }
        if (input == 2) {                          //Both lanes racing selected?
          Lane_Usage = 3;                          //Set lane usage to both lanes
          Str1 = "LN:3";                           //Set lane usage command for both lanes
        }
        myPort.write(Str1);                        //Send lane usage command to Arduino timer
        myPort.write(lf);                          //followed by a line-feed
        delay(10);                                 //Wait for Arduino timer to process
        myPort.write("C");                         //Send 'Continue' command to timer
        myPort.write(lf);                          //followed by a line-feed
        Draw_RaceOptions();                        //Call subroutine to update race option display
      }
      else {
        prompt = "Must be in Ready or Finished state\nto select active lanes";
        showMessageDialog(null,prompt);                   //Display message
      }
    }
    //Test for Double Red option select
    if ((x>1050) && (x<1125) && (y>560) && (y<585)) {
      if (Race_State<2 || Race_State==5) {         //Allow only if not in a staged or racing state //<>//
        DoubleRedflag = !DoubleRedflag;            //Toggle flag
        if (DoubleRedflag) {
          myPort.write("DBR:1");                   //Send Double Red enable command to Arduino
          myPort.write(lf);                        //followed by a line-feed
        }
        else {
          myPort.write("DBR:0");                   //Send Double Red disable command to Arduino
          myPort.write(lf);                        //followed by a line-feed
        }
      Draw_RaceOptions();                          //Call subroutine to update race option display
      }
      else {
        prompt = "Must be in Ready or Finished state\nto select Double Red option";
        showMessageDialog(null,prompt);            //Display message
      }
    }
    //Test for Top Bulb Lock select
    if ((x>1050) && (x<1125) && (y>590) && (y<615)) {
      if (Race_State<2 || Race_State==5) {         //Allow only if not in a staged or racing state
        TopBulbLockflag = !TopBulbLockflag;        //Toggle flag
        if (TopBulbLockflag) {                          //Do if flag is set
          myPort.write("TBL:1");                   //Send Top Bulb Lock enable command to Arduino
          myPort.write(lf);                        //followed by a line-feed
        }
        else {                                     //Do if flag is not set
          myPort.write("TBL:0");                   //Send Top Bulb Lock disable command to Arduino
          myPort.write(lf);                        //followed by a line-feed
        }
        Draw_RaceOptions();                        //Call subroutine to update race option display
      }
      else {
        prompt = "Must be in Ready or Finished state\nto select Top Bulb Lock option";
        showMessageDialog(null,prompt);            //Display message
      }
    }
  }
}

void Clear_Data() {
  //Draw text boxes with curved corners in which the lane times and finish order will be displayed.
  //Note: This will also overwrite (clear) any previous data drawn inside
  fill(100);                                       //Set rectangle color
  rect(90,60,140,30,10);                           //Draw rectangle for Lane1 Racer's Class
  rect(90,100,140,30,10);                          //Draw rectangle for Lane1 Racer's Car#
  rect(90,140,140,30,10);                          //Draw rectangle for Lane1 Dial-In time
  rect(90,180,140,30,10);                          //Draw rectangle for Lane1 RT
  rect(90,220,140,30,10);                          //Draw rectangle for Lane1 60Ft ET
  rect(90,260,140,30,10);                          //Draw rectangle for Lane1 660Ft ET
  rect(90,300,140,30,10);                          //Draw rectangle for Lane1 1320Ft ET
  rect(90,340,140,30,10);                          //Draw rectangle for Lane1 1320Ft Speed
  rect(90,380,140,30,10);                          //Draw rectangle for Lane1 Total time
  rect(90,420,140,30,10);                          //Draw rectangle for Lane1 race status
  rect(90,460,140,30,10);                          //Draw rectangle for Lane1 Margin
  rect(350,60,140,30,10);                          //Draw rectangle for Lane2 Racer's Class
  rect(350,100,140,30,10);                         //Draw rectangle for Lane2 Racer's Car#
  rect(350,140,140,30,10);                         //Draw rectangle for Lane2 Dial-In time
  rect(350,180,140,30,10);                         //Draw rectangle for Lane2 RT
  rect(350,220,140,30,10);                         //Draw rectangle for Lane2 60Ft ET
  rect(350,260,140,30,10);                         //Draw rectangle for Lane2 660Ft ET
  rect(350,300,140,30,10);                         //Draw rectangle for Lane2 1320Ft ET
  rect(350,340,140,30,10);                         //Draw rectangle for Lane2 1320Ft Speed
  rect(350,380,140,30,10);                         //Draw rectangle for Lane2 Total time
  rect(350,420,140,30,10);                         //Draw rectangle for Lane2 race status
  rect(350,460,140,30,10);                         //Draw rectangle for Lane2 Margin
}

void Draw_Racebox() {
  //Draw the Race Count box and current Race number
  //Note: This will also overwrite (clear) any previous data drawn inside the box
  fill(100);                                       //Set rectangle color
  rect(460,550,100,80);                            //and draw it
  textSize(50);                                    //Set text size to 65 pixels
  textAlign(CENTER);                               //Set text justification to center
  fill(255);                                       //Set text color to white
  if (Race_Number > 0) text(Race_Number,510,605);  //Draw race number if > 0
    else text("#",510,605);
}

void Set_Race_State() {
  textAlign(LEFT);                                 //Set text justification to left
  textSize(15);                                    //Set text size to 25 pixels
  fill(150);                                       //Set color to dim circle & text
  ellipse(45,545,15,15);                           //Draw dimmed circle
  text("Not Ready",75,550);                        //Draw dimmed text
  ellipse(45,565,15,15);                           //Draw dimmed circle
  text("Ready",75,570);                            //Draw dimmed text
  ellipse(45,585,15,15);                           //Draw dimmed circle
  text("Staged",75,590);                           //Draw dimmed text
  ellipse(45,605,15,15);                           //Draw dimmed circle
  text("Countdown",75,610);                        //Draw dimmed text
  ellipse(45,625,15,15);                           //Draw dimmed circle
  text("Racing",75,630);                           //Draw dimmed text
  ellipse(45,645,15,15);                           //Draw dimmed circle
  text("Finished",75,650);                         //Draw dimmed text
  switch (Race_State) {                            //Test the race state
    case 0:                                        //Do the following if it is 0
      fill(255,0,0);                               //Set color to red
      ellipse(45,545,15,15);                       //Draw red circle
      text("Not Ready",75,550);                    //Draw red text
      break;
    case 1:                                        //Do the following if it is 1
      fill(245, 255, 0);                           //Set color yellow
      ellipse(45,565,15,15);                       //Draw yellow circle
      text("Ready",75,570);                        //Draw yellow text
      break;
    case 2:                                        //Do the following if it is 2
      fill(255);                                   //Set color white
      ellipse(45,585,15,15);                       //Draw white circle
      text("Staged",75,590);                       //Draw white text
      break;
    case 3:                                        //Do the following if it is 3
      fill(250,175,50);                            //Set color to amber
      ellipse(45,605,15,15);                       //Draw amber circle
      text("Countdown",75,610);                    //Draw amber text
      break;
    case 4:                                        //Do the following if it is 4
      fill(0, 180, 0);                             //Set color to dark green
      ellipse(45,625,15,15);                       //Draw green circle
      text("Racing",75,630);                       //Draw dark green text
      break;
    case 5:                                        //Do the following if it is 5
      fill(0, 0, 200);                             //Set color to blue
      ellipse(45,645,15,15);                       //Draw blue circle
      text("Finished",75,650);                     //Draw blue text
      break;
    default:                                       //Do nothing if it is something else
      //do nothing
      break;
  }
}

void Display_Results() {  //Write race results to their respective text boxes on the screen
  Clear_Data();                                            //Call subroutine to clear old data
  textAlign(CENTER);                                       //Set text justification to center
  fill(255);                                               //Set text color to white
  textSize(30);                                            //Set text size to 30 pixels
  //Display Lane1 values
  text(RClass[0],160,85);                                  //Display Lane1 Class
  text(Carnum[0],160,125);                                 //Display Lane1 Car#
  Str1 = nf(DialinTime[0],0,2);                            //Convert to formatted string
  text(Str1,160,165);                                      //Display Lane1 Dial-in
  if (abs(ReactTime[0]) < 900.0) {                         //Check if valid reaction time value and display
    if (ReactTime[0]<0) fill(0xFFFF0000);                  //Set text color to red if negative
    text(ReactTime[0],160,205);                            //Display Lane1 reaction time
    fill(255);                                             //Set text color back to white
  }
  else text("- - - - -",160,205);                          //Or display dashes
  if (Trk60ftTime[0] < 900.0) text(Trk60ftTime[0],160,245); //Check if valid 60Ft ET and display
    else text("- - - - -",160,245);                        //Or display dashes
  if (Trk660ftTime[0] < 900.0) text(Trk660ftTime[0],160,285); //Check if valid 660Ft ET and display
    else text("- - - - -",160,285);                        //Or display dashes
  if (Trk1320ftTime[0] < 900.0) text(Trk1320ftTime[0],160,325); //Check if valid elapsed time value and display
    else text("- - - - -",160,325);                        //Or display dashes
  if (Speed[0] < 900.0) {                                  //Check if valid 1320Ft speed
    Str1 = nf(Speed[0],0,2);                               //Convert to formatted string
    text(Str1,160,365);                                    //And display it
  }
  else text("- - - -",160,365);                            //Or display dashes
  if (OverallTime[0] < 900.0) text(OverallTime[0],160,405); //Check if valid OT and display
    else text("- - - -",160,405);                          //Or display dashes
  fill(L1Color);                                           //Set Lane1 status color
  text(L1status,160,445);                                  //Display Lane1 status
  fill(255);                                               //Set text color to white
  if (Margin[0] < 900.0) text(Margin[0],160,485);          //Check if valid margin and display
    else text("- - - - -",160,485);                        //Or display dashes
  
  //Display Lane2 values
  text(RClass[1],420,85);                                  //Display Lane2 Class
  text(Carnum[1],420,125);                                 //Display Lane2 Car#
  Str1 = nf(DialinTime[1],0,2);                            //Convert to formatted string
  text(Str1,420,165);                                      //Display Lane2 Dial-in
  if (abs(ReactTime[1]) < 900.0) {                         //Check if valid reaction time value and display
    if (ReactTime[1]<0) fill(0xFFFF0000);                  //Set text color to red if negative
    text(ReactTime[1],420,205);                            //Display Lane1 reaction time
    fill(255);                                             //Set text color back to white
  }
  else text("- - - - -",420,205);                          //Or display dashes
  if (Trk60ftTime[1] < 900.0) text(Trk60ftTime[1],420,245); //Check if valid 60Ft ET and display
    else text("- - - - -",420,245);                        //Or display dashes
  if (Trk660ftTime[1] < 900.0) text(Trk660ftTime[1],420,285); //Check if valid 660Ft ET and display
    else text("- - - - -",420,285);                        //Or display dashes
  if (Trk1320ftTime[1] < 900.0) text(Trk1320ftTime[1],420,325); //Check if valid elapsed time value and display
    else text("- - - - -",420,325);                        //Or display dashes
  if (Speed[1] < 900.0) {                                  //Check if valid 1320Ft speed
    Str1 = nf(Speed[1],0,2);                               //Convert to formatted string
    text(Str1,420,365);                                    //And display
  }
  else text("- - - -",420,365);                            //Or display dashes
  if (OverallTime[1] < 900.0) text(OverallTime[1],420,405); //Check if valid OT and display
    else text("- - - -",420,405);                          //Or display dashes
  fill(L2Color);                                           //Set Lane2 status color
  text(L2status,420,445);                                  //Display Lane2 status
  fill(255);                                               //Set text color to white
  if (Margin[1] < 900.0) text(Margin[1],420,485);          //Check if valid margin and display
    else text("- - - - -",420,485);                        //Or display dashes
}

void Redisplay_RaceResults() { //Get & Redisplay race results for the current selected race
  //Read data from results table
  Carnum[0] = Resultstable.getString(Index1, "L1_Car");          //Get saved Lane1 Car#
  RClass[0] = Resultstable.getString(Index1, "L1_Class");        //Get saved Lane1 race class
  Ln1Foul = Resultstable.getInt(Index1,"L1_Foul");               //Get saved foul status for Lane1
  Brkout1flag = Resultstable.getInt(Index1,"L1_BrkOutflag");     //Get saved Breakout status for Lane1
  DialinTime[0] = Resultstable.getFloat(Index1, "L1_Dialin");    //Get saved dial-in time for Lane1
  ReactTime[0] = Resultstable.getFloat(Index1, "L1_React");      //Get saved reaction time for Lane1
  Trk60ftTime[0] = Resultstable.getFloat(Index1, "L1_60Ft");     //Get saved 60 Ft time for Lane1
  Trk660ftTime[0] = Resultstable.getFloat(Index1, "L1_660Ft");   //Get saved 660 Ft time for Lane1
  Trk1320ftTime[0] = Resultstable.getFloat(Index1, "L1_1320Ft"); //Get saved 1320 Ft time for Lane1
  Speed[0] = Resultstable.getFloat(Index1, "L1_Speed");          //Get saved 1320 Ft speed for Lane1
  OverallTime[0] = Resultstable.getFloat(Index1, "L1_Ovl");      //Get saved overall time for Lane1
  Margin[0] = Resultstable.getFloat(Index1, "L1_Margin");        //Get saved Margin for Lane1
  Win1flag = Resultstable.getInt(Index1, "L1_Winflag");          //Get saved win status for Lane1
  Carnum[1] = Resultstable.getString(Index1, "L2_Car");          //Get saved Lane2 Car#
  RClass[1] = Resultstable.getString(Index1, "L2_Class");        //Get saved Lane2 race class
  Ln2Foul = Resultstable.getInt(Index1,"L2_Foul");               //Get saved foul status for Lane2
  Brkout2flag = Resultstable.getInt(Index1,"L2_BrkOutflag");     //Get saved Breakout status for Lane2
  DialinTime[1] = Resultstable.getFloat(Index1, "L2_Dialin");    //Get saved dial-in time for Lane2
  ReactTime[1] = Resultstable.getFloat(Index1, "L2_React");      //Get saved reaction time for Lane2
  Trk60ftTime[1] = Resultstable.getFloat(Index1, "L2_60Ft");     //Get saved 60 Ft time for Lane2
  Trk660ftTime[1] = Resultstable.getFloat(Index1, "L2_660Ft");   //Get saved 660 Ft time for Lane2
  Trk1320ftTime[1] = Resultstable.getFloat(Index1, "L2_1320Ft"); //Get saved 1320 Ft time for Lane2
  Speed[1] = Resultstable.getFloat(Index1, "L2_Speed");          //Get saved 1320 Ft speed for Lane2
  OverallTime[1] = Resultstable.getFloat(Index1, "L2_Ovl");      //Get saved overall time for Lane2
  Margin[1] = Resultstable.getFloat(Index1, "L2_Margin");        //Get saved Margin for Lane2
  Win2flag = Resultstable.getInt(Index1, "L2_Winflag");          //Get saved win status for Lane2
  TS_date = Resultstable.getString(Index1, "Date");              //Get time slip date stamp 
  TS_time = Resultstable.getString(Index1, "Time");              //Get time slip time stamp
  Set_Racer_Status();                                            //Call subroutine to get racer's status
  Clear_Msgbox();                                                //Clear message text box
  Display_Results();                                             //Call subroutine to display the results
  //ADD STEPS TO CALC OVERALL TIME
}

void Draw_StartRace_Button() {               //Subroutine to display the Start Race select button
  fill(200);                                 //Set rectangle color
  rect(255,600,80,60,10);                    //Draw rectangle
  textSize(25);                              //Set text size to 25 pixels
  textAlign(CENTER);                         //Set text justification to center
  fill(150);                                 //Set text color to grey (Disabled)
  if (Race_State == 2) fill(0);              //Set text color to black (Enabled) if timer in the 'Staged' state
  text("Start",295,625);                     //Display "Start" inside text box top
  text("Race",295,655);                      //Display "Race" inside text box bottom
}

void Draw_NextRace_Button() {                //Subroutine to display the Next Race select button
  fill(200);                                 //Set rectangle color
  rect(255,535,80,60,10);                    //Draw rectangle
  textSize(25);                              //Set text size to 25 pixels
  textAlign(CENTER);                         //Set text justification to center
  fill(150);                                 //Set text color to grey (Disabled)
  if (NextRaceflag) fill(0);                 //Set text color to black (Enabled) if NextRaceflag = TRUE
  text("Next",295,560);                      //Display "Next" inside text box top
  text("Race",295,590);                      //Display "Race" inside text box bottom
}

void Set_Racer_Status() {                    //Set winner based on win, foul & breakout Status
  if (Win1flag==1 && Win2flag==0) {          //Check if Lane1 is the Winner
    if (Ln1Foul==0 && Ln2Foul==0) {          //Do if no lanes fouled
      L1Color = 0xFF00FF00;                  //Set text color to green
      L1status = "WINNER";                   //Set Lane1 status as the winner
      L2Color = 0xFFFFFF00;                  //Set text color to yellow
      L2status = "- - -";                    //Set Lane2 status as the looser
      if (Brkout2flag == 1) {                //Do if Lane2 broke out
        L2status = "BREAKOUT";
      }
    }
    if (Ln1Foul==0 && Ln2Foul==1) {          //Do if only Lane2 fouled
      L1Color = 0xFF00FF00;                  //Set text color to green
      L1status = "WINNER";                   //Set Lane1 status as the winner
      if (OverallTime[1]<OverallTime[0]) {   //Did Lane1 win by default?
        L1status = "WINNER*";                //Set Lane1 status as the default winner
      }
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = "DISQ";                     //Set Lane2 status as disqualified
    }
    if (Ln1Foul==2) {                        //Do if Lane1 won by default
      L1Color = 0xFF00FF00;                  //Set text color to green
      L1status = "WINNER*";                  //Set Lane1 status as the default winner
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = "DISQ";                     //Set Lane2 status as disqualified
    }
  }
  if (Win1flag==0 && Win2flag==1) {          //Check if Lane2 is the Winner
    if (Ln1Foul==0 && Ln2Foul==0) {          //Do if no lanes fouled
      L1Color = 0xFFFFFF00;                  //Set text color to yellow
      L1status = "- - -";                    //Set Lane1 status as the looser
      if (Brkout1flag == 1) {                //Do if Lane1 broke out
        L1status = "BREAKOUT";
      }
      L2Color = 0xFF00FF00;                  //Set text color to green
      L2status = "WINNER";                   //Set Lane2 status as the winner
    }
    if (Ln1Foul==1 && Ln2Foul==0) {          //Do if only Lane1 fouled
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = "DISQ";                     //Set Lane1 status as Disqualified
      L2Color = 0xFF00FF00;                  //Set text color to green
      L2status = "WINNER";                   //Set Lane2 status as the winner
      if (OverallTime[0]<OverallTime[1]) {   //Did Lane2 win by default?
        L2status = "WINNER*";                //Set Lane1 status as the default winner
      }
    }
    if (Ln2Foul==2) {                        //Do if Lane2 won by default
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = "DISQ";                     //Set Lane1 status as disqualified
      L2Color = 0xFF00FF00;                  //Set text color to green
      L2status = "WINNER*";                  //Set Lane2 status as the default winner
    }
  }
  if (Win1flag==0 && Win2flag==0) {          //Check if a Draw (Tie)
    if (Ln1Foul==0 && Ln2Foul==0) {          //Do if no lanes fouled
      L1Color = 0xFFFFFF00;                  //Set text color to yellow
      L1status = "- - -";                    //Set Lane1 status to a draw
      if (Brkout1flag == 1) {                //Do if Lane1 broke out
        L1status = "BREAKOUT";
      }
      L2Color = 0xFFFFFF00;                  //Set text color to yellow
      L2status = "- - -";                    //Set Lane2 status to a draw
      if (Brkout2flag == 1) {                //Do if Lane2 broke out
        L2status = "BREAKOUT";
      }
    }
    if (Ln1Foul==1 && Ln2Foul==0) {          //Do if only Lane1 fouled
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = "DISQ";                     //Set Lane1 status as disqualified
      L2Color = 0xFFFFFF00;                  //Set text color to yellow
      L2status = "- - -";                    //Set Lane2 status to a draw
      if (Brkout2flag == 1) {                //Do if Lane2 broke out
        L2status = "BREAKOUT";
      }
    }
    if (Ln1Foul==0 && Ln2Foul==1) {          //Do if only Lane2 fouled
      L1Color = 0xFFFFFF00;                  //Set text color to yellow
      L1status = "- - -";                    //Set Lane1 status to a draw
      if (Brkout1flag == 1) {                //Do if Lane1 broke out
        L1status = "BREAKOUT";
      }
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = "DISQ";                     //Set Lane2 status as disqualified
    }
    if (Ln1Foul>0 && Ln2Foul>0) {            //Do if both lanes fouled
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = "DISQ";                     //Set Lane1 status as disqualified
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = "DISQ";                     //Set Lane2 status as disqualified
    }
  }
}

void Clear_Msgbox() {
  //Draw rectangle message box in which the error messages will be displayed
  //Note: This will also overwrite (clear) any previous data drawn inside
  fill(100);                                 //Set rectangle color
  rect(65,670,1100,27,10);                   //Draw it
}

void Disp_Message(int Msg_Number) {
  Clear_Msgbox();                            //Call Clear Message Box subroutine
  textAlign(LEFT);                           //Set text justification to left
  textSize(23);                              //Set text size to 23 pixels
  fill(255);                                 //Set color to white
  switch (Msg_Number) {                      //Test the message number
    case 0:                                  //Do nothing if it is 0
      //do nothing
      break;
    case 1:                                  //Do the following if it is 1
      text(Msg1,70,690);                     //Display message
      break;
    case 2:                                  //Do the following if it is 2
      text(Msg2,70,690);                     //Display message
      break;
    case 3:                                  //Do the following if it is 3
      text(Msg3,70,690);                     //Display message
      break;
    case 4:                                  //Do the following if it is 4
      text(Msg4,70,690);                     //Display message
      break;
    case 5:                                  //Do the following if it is 5 (Not yet implemented)
      text(Msg5,70,690);                     //Display message
      break;
    case 6:                                  //Do the following if it is 6 (Not yet implemented)
      text(Msg6,70,690);                     //Display message
      break;
    case 7:                                  //Do the following if it is 7
      text(Msg7,70,690);                     //Display message
      break;
    default:                                 //Do nothing if it is something else
      //do nothing
      break;
  }
}

float Truncate(float Value) {                //Function to truncate value to 3 decimal places
  return round(Value*1000.0f)/1000.0f;
}

float RoundValue(float Value, int p) {
  i = int(Value * pow(10,p));
  return round(Value*100.0f)/100.0f;
}

void Get_datetime() {                        //Function: Get PC Date/Time for the Time Slip
  Yr = year();                               //Get year value from PC clock
  Mo = month();                              //Get month value from PC clock
  Day = day();                               //Get day value from PC clock
  Hr = hour();                               //Get hour value from PC clock
  Min = minute();                            //Get minute value from PC clock
  TS_date = nf(Mo,2)+"/"+nf(Day,2)+"/"+nf(Yr,4); //Build date string
  TS_time = nf(Hr,2)+":"+nf(Min,2);              //Build time string
}

void RacefileSelect(File selection) {
  if (selection == null) Canceledflag = true;
  else {
    raceFilename = selection.getAbsolutePath();
    Racefile_Selected = true;
  }
}

void keyPressed() {                           //Key Press interrupt routine
  if (keyCode==112) {                         //Check if F1 key pressed to select Xmas Tree lamp test
    if (Race_State<2 || Race_State==5) {      //Allow only if not in a staged or racing state
      myPort.write("LT");                     //Send lamp test command to timer
      myPort.write(lf);                       //followed by a line-feed
    }
  }
}

void Draw_RaceOptions() {                     //Draw race option section of race page
  fill(Rx, Gx, Bx);                           //Set background color
  noStroke();                                 //Disable rectangle outline
  rect(780,525,80,95);                        //Clear current values
  rect(1050,525,80,95);                       //Clear current values
  stroke(0);                                  //Enable rectangle outline
  textAlign(CENTER);                          //Set text justification to center
  textSize(20);                               //Set text size to 20 pixels
  fill(240,280,80);                           //Set color to yelllow
  rect(780,530,75,25);                        //Draw text cell for Tree Mode
  rect(780,560,75,25);                        //Draw text cell for Auto/Man Start
  rect(780,620,100,25);                       //Draw text cell for Timeslip Option
  rect(1050,530,75,25);                       //Draw text cell for Active Lanes
  fill(0);                                    //Set color to black
  //Process Tree Mode Option
  if (Tree_Mode.equals("M:0")) text("STD",817,550);
  if (Tree_Mode.equals("M:1")) text("PRO",817,550);
  if (Tree_Mode.equals("M:2")) text("OUTLAW",817,550);
  //Process Auto Start Option
  if (Autostartflag) text("AUTO",817,580);    //Display text
    else text("MAN",817,580);                 //Display text
  //Process Handicap Enable Option
  if (HandiCapEnblflag) {
    fill(0,200,00);                          //Set color to green
    rect(780,590,75,25);                     //Draw text cell for Double Red
    fill(0);                                 //Set color to black
    text("Enbl",817,610);                    //Display text
  }
  else {
    fill(200,0,0);                           //Set color to red
    rect(780,590,75,25);                     //Draw text cell for Double Red
    fill(0);                                 //Set color to black
    text("Dsbl",817,610);                    //Display text
  }
  //Process Individual Timeslip Option
  if (IndyTSflag) text("Individual",830,640); //Display text
    else text("Normal",830,640);              //Display text
  //Process Active Lanes selection
  if (Lane_Usage==1) text("Lane 1",1087,550); //Display Text
  if (Lane_Usage==2) text("Lane 2",1087,550); //Display Text
  if (Lane_Usage==3) text("Both",1087,550);   //Display Text
  //Process Double Red Option
  if (DoubleRedflag) {
    fill(0,200,00);                           //Set color to green
    rect(1050,560,75,25);                     //Draw text cell for Double Red
    fill(0);                                  //Set color to black
    text("On",1087,580);                      //Display text
  }
  else {
    fill(200,0,0);                            //Set color to red
    rect(1050,560,75,25);                     //Draw text cell for Double Red
    fill(0);                                  //Set color to black
    text("Off",1087,580);                     //Display text
  }
  //Process Top Bulb Lock Option
  if (TopBulbLockflag) {
    fill(0,200,00);                           //Set color to green
    rect(1050,590,75,25);                     //Draw text cell for Top Bulb Lock
    fill(0);                                  //Set color to black
    text("On",1087,610);                      //Display text
  }
  else {
    fill(200,0,0);                            //Set color to red
    rect(1050,590,75,25);                     //Draw text cell for Top Bulb Lock
    fill(0);                                  //Set color to black
    text("Off",1087,610);                     //Display text
  }
}

void Create_Timeslip(int lane) {             //Create dual or individual time slip text file
  TS_output = createWriter(TSfileName);      //Open time slip text file in the sketch directory
  Str1 = centerText(TS_Header1);             //Get & center time slip header line 1
  TS_output.println(Str1);                   //Write header line 1 to the file
  Str1 = centerText(TS_Header2);             //Get & center time slip header line 2
  TS_output.println(Str1);                   //Write header line 2 to the file
  TS_output.println("=================================");
  Str1 = "     Date: "+TS_date+"  "+TS_time; //Create the date/time string
  TS_output.println(Str1);                   //Write the date & time to the file header
  TS_output.println("            Race# "+nf(Race_Number,3)); //Write race# to the file header
  if (lane<3) { //DO INDIVIDUAL TIMESLIP
    i = lane - 1;                            //Set parameter index
    Str1 = "     Lane: " + nf(lane);         //Format Lane number string
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "     Class...... " + resizeText(RClass[i],10);  //Format Class line
    TS_output.println(Str1); //And write it to the file
    Str1 = "     Car#....... " + resizeText(Carnum[i],10);  //Format Car# line
    TS_output.println(Str1); //And write it to the file
    if (lane==1 && Brkout1flag==1) {         //Do if Lane1 broke out
      TS_output.println("     BREAKOUT");    //Write breakout line to the file
    }
    if (lane==2 && Brkout2flag==1) {         //Do if Lane2 broke out
      TS_output.println("     BREAKOUT");    //Write breakout line to the file
    }
    Str1 = "     Dialin..... " + convertSpecial(DialinTime[i],2); //Format dial-in time line
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "     R/T........ " + convertSpecial(ReactTime[i],3); //Format reaction time line
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "     60 Ft...... " + convertSpecial(Trk60ftTime[i],3); //Format 60 Ft time line
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "    1/8 ET...... " + convertSpecial(Trk660ftTime[i],3); //Format 1/8 mile time line
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "    1/4 ET...... " + convertSpecial(Trk1320ftTime[i],3); //Format 1/4 mile time line
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "   1/4 MPH...... " + convertSpecial(Speed[i],2); //Format 1/4 mile speed line
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "    Status......    ----";       //Set default Lane1 Status text
    if (i==0) {
      if (L1status.equals("WINNER")) Str1 = "    Status......     WINNER ";  //Set Lane1 Status text
      if (L1status.equals("WINNER*")) Str1 = "    Status......     WINNER*"; //Set Lane1 Status text
      if (L1status.equals("DISQ")) Str1 = "    Status......      DISQ  ";    //Set Lane1 Status text
    }
    if (i==1) {
      if (L2status.equals("WINNER")) Str1 = "    Status......     WINNER ";  //Set Lane1 Status text
      if (L2status.equals("WINNER*")) Str1 = "    Status......     WINNER*"; //Set Lane1 Status text
      if (L2status.equals("DISQ")) Str1 = "    Status......      DISQ  ";    //Set Lane1 Status text
    }
    TS_output.println(Str1);                 //And write it to the file
    Str1 = "    Margin...... " + convertSpecial(Margin[i],3); //Format margin time line
    TS_output.println(Str1);                 //And write it to the file
  }
  else { //DO DUAL TIMESLIP
    TS_output.println(" -Lane 1-               -Lane 2-"); //Write lane header to the file
    Str1 = resizeText(RClass[0],10);         //Call function to format Lane1 Car#
    Str2 = resizeText(RClass[1],10);         //Call function to format Lane2 Car#
    Str3 = Str1+"    Class   "+Str2;         //Format Racer Class line
    TS_output.println(Str3);                 //Write Racer Class line to the file
    Str1 = resizeText(Carnum[0],10);         //Call function to format Lane1 Car#
    Str2 = resizeText(Carnum[1],10);         //Call function to format Lane2 Car#
    Str3 = Str1+"     Car#   "+Str2;         //Format Racer Car# line
    TS_output.println(Str3);                 //Write Racer Car# line to the file
    if (Brkout1flag==1 && Brkout2flag==0) {  //Do if Lane1 broke out
      Str1 = "  BREAKOUT";                   //Format Breakout line
      TS_output.println(Str1);               //Write breakout line to the file
    }
    if (Brkout1flag==0 && Brkout2flag==1) {  //Do if Lane2 broke out
      Str1 = "                        BREAKOUT"; //Format Breakout line
      TS_output.println(Str1);               //Write breakout line to the file
    }
    if (Brkout1flag==1 && Brkout2flag==1) {  //Do if both lanes broke out
      Str1 = "  BREAKOUT              BREAKOUT"; //Format Breakout line
      TS_output.println(Str1);               //Write breakout line to the file
    }
    Str1 = convertSpecial(DialinTime[0],2);  //Call function to convert to 10-char string
    Str2 = convertSpecial(DialinTime[1],2);  //Call function to convert to 10-char string
    Str3 = Str1+"    Dialin  "+Str2;         //Format dial-in time line string
    TS_output.println(Str3);                 //Write dial-in time line to the file
    Str1 = convertSpecial(ReactTime[0],3);   //Call function to convert to 10-char string
    Str2 = convertSpecial(ReactTime[1],3);   //Call function to convert to 10-char string
    Str3 = Str1+"     R/T    "+Str2;         //Format react time line string
    TS_output.println(Str3);                 //Write reaction time line to the file
    Str1 = convertSpecial(Trk60ftTime[0],3); //Call function to convert to 10-char string
    Str2 = convertSpecial(Trk60ftTime[1],3); //Call function to convert to 10-char string
    Str3 = Str1+"    60 Ft   "+Str2;         //Format 60 Ft time line string
    TS_output.println(Str3);                 //Write 60 Ft time line to the file
    Str1 = convertSpecial(Trk660ftTime[0],3); //Call function to convert to 10-char string
    Str2 = convertSpecial(Trk660ftTime[1],3); //Call function to convert to 10-char string
    Str3 = Str1+"    1/8 ET  "+Str2;         //Format 660 Ft time line string
    TS_output.println(Str3);                 //Write 660 Ft time line to the file
    Str1 = convertSpecial(Trk1320ftTime[0],3); //Call function to convert to 10-char string
    Str2 = convertSpecial(Trk1320ftTime[1],3); //Call function to convert to 10-char string
    Str3 = Str1+"    1/4 ET  "+Str2;         //Format 1/4 mile ET line string
    TS_output.println(Str3);                 //Write 1/4 mile ET line to the file
    Str1 = convertSpecial(Speed[0],2);       //Call function to convert to 10-char string
    Str2 = convertSpecial(Speed[1],2);       //Call function to convert to 10-char string
    if (MetricFlag) Str3 = Str1+"   1/4 Km/h "+Str2; //Format speed line string for Km/h
     else Str3 = Str1+"    1/4 MPH "+Str2;   //Format 1/4 mile speed line string for MPH
    TS_output.println(Str3);                 //Write 1/4 mile speed line to the file
    Str1 = "   - - - -";                     //Set default Lane1 Status text
    Str2 = "- - - -";                        //Set default Lane2 Status text
    if (L1status.equals("WINNER")) Str1 = "    WINNER"; //Set Lane1 Status text
    if (L1status.equals("WINNER*")) Str1 = "   WINNER*"; //Set Lane1 Status text
    if (L1status.equals("DISQ")) Str1 = "     DISQ ";   //Set Lane1 Status text
    if (L2status.equals("WINNER")) Str2 = " WINNER";    //Set Lane2 Status text
    if (L2status.equals("WINNER*")) Str2 = "WINNER*";   //Set Lane2 Status text
    if (L2status.equals("DISQ")) Str2 = "  DISQ";       //Set Lane2 Status text
    Str3 = Str1+"    Status     "+Str2;                 //Build Status line
    TS_output.println(Str3);                 //Write status line to the file
    Str1 = convertSpecial(Margin[0],3);      //Call function to convert to 10-char string
    Str2 = convertSpecial(Margin[1],3);      //Call function to convert to 10-char string
    Str3 = Str1+"    Margin  "+Str2;         //Format margin line string
    TS_output.println(Str3);                 //Write margin line to the file
  }
  TS_output.println("=================================");
  Str1 = centerText(TS_Footer1);             //Get & center time slip footer line 1
  TS_output.println(Str1);                   //Write footer line 1 to the file
  Str1 = centerText(TS_Footer2);             //Get & center time slip footer line 2
  TS_output.println(Str1);                   //Write footer line 2 to the file
  TS_output.println(" ");                    //Write blank line to the file
  TS_output.flush();                         //Writes the remaining data to the file
  TS_output.close();                         //Finishes & closes the file
}

String convertSpecial(float valIn, int d) {  //Function to convert float to 10-character string
  String Strx;                               //Define variable Strx
  if (valIn < 900.0) {
    Strx = nf(valIn,0,d);                    //Convert floating input value to string w/ 'd' decimal palces
    while(Strx.length()<10) {                //Loop while string length is < 10 characters
      Strx = " " + Strx;                     //Add a space character to front of string
    }
  }
  else Strx = "   - - - -";
  return Strx;                               //Return string to calling routine
}

String centerText(String Strg) {             //Subroutine to center time slip text
  String Strx = "";                          //Define variable Strx
  j = (32 - Strg.length())/2;                //Calculate number of spaces needed to center text
  for (i=0; i<j; i++) {                      //Loop as required
    Strx = Strx + " ";                       //and add another space to the string
  }
  Strx = Strx + Strg;                        //Add string of spaces to front of input string
  return Strx;                               //Return string to calling routine
}

String resizeText(String Strg, int l) {
  if (Strg.length()>l) {                     //Check if longer than specified length
    Strg = Strg.substring(0,l);              //And trim to specified length if so
  }
  while(Strg.length()<l) {                   //Loop if less than specified length
    Strg = " " + Strg;                       //Add a space in front
  }
  return Strg;
}


String Resize_RacerID(String Strg) {         //Function to center Racer ID text string
  if (Strg.length()>6) {                     //Check if longer than 6 characters
    Strg = Strg.substring(0,6);              //And trim to a length of 6 if so
  }
  while(Strg.length()<6) {                   //Loop if less than 6 characters
    Strg = " " + Strg + " ";                 //Add a space both in front and behind
    if (Strg.length()>6) {                   //Check if it's now longer than 6 characters
      Strg = Strg.substring(1,7);            //And trim leading space if so
    }
  }
  return Strg;
}

void Init_Setup_Page() {
  background(200);                           //Clear screen with Setup screen background color
  textAlign(CENTER);                         //Set text justification to center
  textSize(40);                              //Set text size to 40 pixels
  fill(255, 0, 170);                         //Set Title color
  text("USER MODIFIABLE PARAMETERS",600,40); //Display Title
  image(logo, 1080, 640);                    //Dsplay logo in lower right corner
  fill(0);                                   //Set color to black
  text("Use this screen to:",600,80);        //Display text
  textSize(30);                              //Set text size to 30 pixels
  text("(1) Select timer COM port, and (2) Change user modifiable parameters",600,120); //Display text
  textSize(20);                              //Set text size to 20 pixels
  text("To change a parameter, click on the parameter value. Click on the Proceed To Race Page button when done.",600,160); //Display text
  textAlign(RIGHT);                          //Set text justification to right
  text("Display Title:",300,270);            //Display text
  text("TS Header Line1:",300,320);          //Display text
  text("TS Header Line2:",300,370);          //Display text
  text("TS Footer Line1:",300,420);          //Display text
  text("TS Footer Line2:",300,470);          //Display text
  text("TS Auto-Print Copies:",300,520);     //Display text
  text("Race Timeout (Sec):",300,570);       //Display text
  text("Speed Display:",300,620);            //Display text
  fill(150);                                 //Set rectangle color
  rect(180,180,120,60,10);                   //Draw rectangle for COMM button
  rect(520,610,160,60,10);                   //Draw rectangle for Proceed button
  textSize(25);                              //Set text size to 25 pixels
  textAlign(CENTER);                         //Set text justification to center
  fill(0);                                   //Set text color to black
  text("SELECT",240,203);                    //Display text
  text("COM PORT",240,233);                  //Display text
  text("Proceed To",600,633);                //Display text
  text("Race Page",600,663);                 //Display text
  Draw_User_Parameters();                    //Call subroutine to draw user parameters
}

void Draw_User_Parameters() {
  fill(0);                                   //Set rectangle color
  rect(320,245,600,30);                      //Draw rectangle for Display Title
  rect(320,295,400,30);                      //Draw rectangle for TS Header Line1
  rect(320,345,400,30);                      //Draw rectangle for TS Header Line2
  rect(320,395,400,30);                      //Draw rectangle for TS Footer Line1
  rect(320,445,400,30);                      //Draw rectangle for TS Footer Line2
  rect(320,495,50,30);                       //Draw rectangle for TS Copies
  rect(320,545,50,30);                       //Draw rectangle for Timeout seconds
  rect(320,595,80,30);                       //Draw rectangle for Speed selection
  fill(255);                                 //Set color to white
  textSize(25);                              //Set text size to 25 pixels
  textAlign(LEFT);                           //Set text justification to left
  text(Disp_Title,325,270);                  //Draw display title
  text(TS_Header1,325,320);                  //Draw TS Header Line1 text
  text(TS_Header2,325,370);                  //Draw TS Header Line2 text
  text(TS_Footer1,325,420);                  //Draw TS Footer Line1 text
  text(TS_Footer2,325,470);                  //Draw TS Footer Line2 text
  textAlign(CENTER);                         //Set text justification to center
  text(Autoprintcopies,345,520);             //Draw TS number of copies
  text(timeout,345,570);                     //Draw timeout value
  if (MetricFlag) text("Km/h",360,620);      //Draw Speed display value
  else text("MPH",360,620);                  //Draw Speed display value
}

void Init_Race_Page() {                      //Initialize the Racing page
  Rx = 140;                                  //Set background color red hue
  Gx = 200;                                  //Set background color green hue
  Bx = 230;                                  //Set background color blue hue
  background(Rx, Gx, Bx);                    //Set display area background color
  //Draw display header and legend info
  textAlign(CENTER);                         //Set text justification to center
  fill(255, 0, 170);                         //Set Title color
  textSize(30);                              //Set text size to 30 pixels
  text(Disp_Title,600,25);                   //Display Title
  fill(0);                                   //Set color to black
  textSize(15);                              //Set text size to 25 pixels
  text("TS Select",540,50);                  //Display Timeslip selct box header
  textSize(25);                              //Set text size to 25 pixels
  text("Lane 1",160,55);                     //Display Lane1 legend
  text("Lane 2",420,55);                     //Display Lane2 legend
  text("Class",290,85);                      //Display Class legend
  text("Car#",290,125);                      //Display Car# legend
  text("Dial In",290,165);                   //Display Racer's Dial-in legend
  text("RT",290,205);                        //Display Reaction Time legend
  text("60' ET",290,245);                    //Display 60 Ft Time legend
  text("1/8 ET",290,285);                    //Display 660 Ft Time legend
  text("1/4 ET",290,325);                    //Display 1320 Ft Time legend
  if (MetricFlag) text("1/4 Km/h",290,365);  //Display 1320 Ft speed legend for Km/h
  else text("1/4 MPH",290,365);              //Display 1320 Ft speed legend for MPH
  text("Ov'l Time",290,405);                 //Display overall time legend
  text("Status",290,445);                    //Display lane Status legend
  text("Margin",290,485);                    //Display lane Margin legend
  textAlign(LEFT);                           //Set text justification to left
  text("Msg:",15,690);                       //Display message box legend
  fill(Rx, Gx, Bx);                          //Set color same as background
  rect(15,510,150,155,10);                   //Draw rectangle for Race Status box
  rect(170,510,425,155,10);                  //Draw rectangle for Race Control box
  rect(170,510,80,155,10);                   //Draw rectangle for Reset Timer box
  fill(0);                                   //Set color back to black
  textSize(20);                              //Set text size to 30 pixels
  text("Race Status",45,530);                //Display Race Status box legend
  text("Race Control",300,530);              //Display Race Control box legend
  rect(610,50,5,600);                        //Draw a verticle divider line 5 pixels wide
  fill(255, 0, 0);                           //Set color to Red
  ellipse(210,590,40,40);                    //Draw Reset Button
  textAlign(CENTER);                         //Set text justification to center
  text("Timer",210,565);                     //Display Reset Button legend
  text("Reset",210,630);                     //Display Reset Button legend
  fill(0,0,255);                             //Set color to Blue
  ellipse(400,590,40,40);                    //Draw Print Timeslip button
  text("Print",400,565);                     //Display Print Timeslip button legend
  text("Timeslip",400,630);                  //Display Print Timeslip button legend
  fill(0);                                   //Set color back to black
  text("Race #",530,545);                    //Display Race Box text header
  fill(100);                                 //Set rectangle color
  rect(560,550,30,80);                       //and draw Up/Down increment box
  fill(0,0,255);                             //Set color to Blue
  triangle(560, 580, 590, 580, 575, 550);    //Draw Up Arrow
  triangle(560, 600, 590, 600, 575, 630);    //Draw Down Arrow
  //Draw Roster stuff
  textSize(30);                              //Set text size to 40 pixels
  fill(0);                                   //Set color to black
  text("Roster",910,50);                     //Draw roster table title
  text("Race Control Options",910,490);      //Draw race control options title
  textSize(20);                              //Set text size to 20 pixels
  text("Click on Driver No. to assign to a lane",910,70); //Draw roster table text
  text("Click on box to change selection",910,510); //Draw race control options text
  text("No.",670,95);                        //Draw column legend
  text("Name",790,95);                       //Draw column legend
  text("Car#",925,95);                       //Draw column legend
  text("Class",1010,95);                     //Draw column legend
  text("Dial-In",1100,95);                   //Draw column legend
  fill(240,250,80);                          //Set color to yelllow
  rect(650,355,115,25);                      //Draw text cell
  rect(785,355,115,25);                      //Draw text cell
  rect(920,355,115,25);                      //Draw text cell
  rect(1055,355,115,25);                     //Draw text cell
  fill(0);                                   //Set color to black
  text("Load Roster",707,375);               //Draw text
  text("Add Driver",842,375);                //Draw text
  text("Delete Driver",977,375);             //Draw text
  text("Save Roster",1112,375);              //Draw text
  textAlign(LEFT);                           //Set text justification
  text("Tree Mode:",650,550);                //Display tree mode header
  text("Start Mode:",650,580);               //Display stert mode header
  text("Handicap Enbl:",650,610);            //Display Handicap Enable header
  text("Timeslip Print:",650,640);           //Display Individual Timeslips header
  text("Active Lanes:",900,550);             //Display Active Lane header
  text("Double Red:",900,580);               //Display Double Red header
  text("Top Bulb Lock:",900,610);            //Display Top Bulb Lock header
  
  //Initialize text & message boxes by calling their respective subroutines
  Clear_Data();                              //Subroutine to initialize lane time text boxes
  Clear_Msgbox();                            //Subroutine to initialize Message box
  Draw_Racebox();                            //Subroutine to initialize Race Count box
  Set_Race_State();                          //Subroutine to initialize Race Status box
  Draw_NextRace_Button();                    //Call subroutine to draw the Next Race button
  Draw_StartRace_Button();                   //Call subroutine to draw the Start Race button
  Draw_RosterTable();                        //Draw roster table
  Draw_RaceOptions();                        //Draw race options section
  
  //Prior saved race continuation select option
    Str1 = "RACE EVENT SELECT";
    prompt = "Do you wish to continue from a previously saved race event?";
    int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION);  //Prompt: 0=yes, 1=no
    if (input == 0) {                                         //Prompt user for race event file to use
      Canceledflag = false;                                   //Set file select cancel flag to false
      selectInput("Select the race event file to use:", "RacefileSelect"); //Pop-up to select race plan file
      while (!Racefile_Selected) {                            //Wait until it is selected
        delay(1000); 
        if (Canceledflag) break;                              //True if file selection was canceled
      }
      if (raceFilename.indexOf("Race_Results") >= 0) {        //Check if correct file type was selected
        Resultstable = loadTable(raceFilename, "header");     //Load previously saved race event into table
        Race_Number = Races_Ran = Resultstable.getRowCount(); //Initialize race number indexes
        RaceContinueflag = true;                              //Set race continuation flag
        Draw_Racebox();                                       //Display current race number
        Index1 = Race_Number - 1;                             //Set table index pointer
        Redisplay_RaceResults();                              //Call subroutine to display results
      }
      else {
        showMessageDialog(null,"INCORRECT FILE SELECTED\nMUST RESTART PROGRAM");
        println(raceFilename);
        exit();
      }
    }
    if (!RaceContinueflag) Disp_Message(4);  //Call subroutine to display initial race start message
      else Disp_Message(3);                  //Call subroutine to display race continuation message
    //Draw_Settings();
}

void Draw_RosterTable() {
  fill(Rx, Gx, Bx);                          //Set default background color for all table cells
  noStroke();                                //Disable object draw outline
  rect(650,100,520,250);                     //Clear table area
  stroke(0);                                 //Enable object draw outline
  j=100;                                     //Set table starting y-coordinate
  k=25;                                      //Set default row height
  for (i=0; i<10; i++) {                     //Loop to draw 10 rows
    rect(650,j,40,k);                        //Draw Index# cell
    rect(690,j,200,k);                       //Draw Driver Name cell
    rect(890,j,70,k);                        //Draw Car# cell
    rect(960,j,100,k);                       //Draw Class cell
    rect(1060,j,80,k);                       //Draw Dial-in Time cell
    j = j + k;                               //Bump the y-ccordinate for the next row
  }
  rect(1140,100,30,250);                     //Draw Up/Down Arrow area
  fill(200);                                 //Set color to gray
  triangle(1140,130,1155,100,1170,130);      //Draw Table up-arrow
  triangle(1140,320,1155,350,1170,320);      //Draw Table down-arrow
  fill(0);                                   //Set color to black
}

void Load_Roster() {
  selectInput("Select the Roster file to use:", "RosterSelect"); //Open window to select roster file
  while (!RosterFile_Selected) {             //Wait for selection to be made
    delay(1000); 
  }
  Rostertable = loadTable(RosterFilename, "header"); //Load roster table from file
  if (Rostertable.getRowCount()<2) {         //Check for valid roster file
    prompt = "Invalid Roster File - Must have at least 2 racers.\nFix & restart program.";
    showMessageDialog(null,prompt);          //Display the prompt for acknowledgement
    exit();                                  //And exit the program
  }
  if (Rostertable.getRowCount()>99) {        //Check for valid roster file
    prompt = "Invalid Roster File - Can't have more than 99 racers.\nFix & restart program.";
    showMessageDialog(null,prompt);          //Display the prompt for acknowledgement
    exit();                                  //And exit the program
  }
  RosterIndex = 0;
  Display_Roster();
}

void RosterSelect(File selection) {          //Used in roster file selection popup
  if (selection == null) RosterFile_Selected = false;
  else {
    RosterFilename = selection.getAbsolutePath();
    RosterFile_Selected = true;
  }
}

void Display_Roster() {
  Draw_RosterTable();                        //Clear & redraw roster table
  fill(0);                                   //Set color to black
  textSize(20);                              //Set text size to 20 pixels
  textAlign(CENTER);                         //Set text justification to center
  n = Rostertable.getRowCount();             //Get number of roster entries
  j=120;                                     //Set table starting y-coordinate
  for (i=RosterIndex; i<RosterIndex+10; i++) { //Loop to draw up to 10 rows
    if (i<n) {                               //Only display if <= to table row count
      text(i+1,670,j);                       //Draw table entry number
      Str1 = Rostertable.getString(i, "Name"); //Get racer name
      text(Str1,790,j);                      //and display it
      Str1 = Rostertable.getString(i, "CarNo");//Get racer car number
      text(Str1,925,j);                      //and display it
      Str1 = Rostertable.getString(i, "Class");//Get racer class
      text(Str1,1010,j);                     //and display it
      Str1 = nf(Rostertable.getFloat(i, "Dialin"),0,2); //Get racer dialin time
      text(Str1,1100,j);                     //and display it
      j = j + k;
    }
  }
}

void Select_Comm() {
  ComPort_Avail = false;                     //Default COM port available to false
  //Check & Open the serial port at 9600 baud and buffer data till linefeed received
  try {                                      //Use the 'try' & 'catch' exception handling process
    i = Serial.list().length;                //Get the number of open ports
    if (i != 0) {                            //Do if at least one port is open
      if (i >= 2) {                          //Do if more than one open port
        //Create a list of found COM ports from which the user can pick
        for (j = 0; j < i; j++) {
          COMlist += char(j+'a') + " = " + Serial.list()[j];
          if (j+1 < i) COMlist += ",  ";
        }
        COMx = showInputDialog("Which COM port is correct? (a,b,..):\n"+COMlist);
        if (COMx == null) ComPort_Avail = false;;
        if (COMx.isEmpty()) ComPort_Avail = false;;
        i = int(COMx.toLowerCase().charAt(0) - 'a') + 1;
      }
      Portname = Serial.list()[i-1];         //Get the selected port name
      myPort = new Serial(this, Portname, 9600); //Activate COM port at 9600 baud
      myPort.bufferUntil('\n');              //Buffer until Line Feed appears
      prompt = "COM Port Selected: " + Portname; //Construct COM message & COM Port
      showMessageDialog(null,prompt);        //Display the prompt for acknowledgement
      ComPort_Avail = true;                  //Set Com Port avail flag
    }
    else {
      showMessageDialog(null,"No open/connected COM ports detected.\nFix problem & try again");
    }
  }
  catch (Exception e) { //Error catch routine. Print the type of error
    showMessageDialog(null,"COM port is not available\n(may be in use by another program)");
    //println("Error:", e);                  //For debug only
    //exit();                                //For debug only
  }
}
