/*====================================================================================================*
* 2-Lane Drag Race Timer                                                     Version 14 - 13 Jan 2024 *
* Created by: Bill van Leeuwen (email: Billv923@outlook.com) for the Arduino MEGA microcontroller     *
* Version 14 provides the following features:                                                         *
*   1. Handicap racing capabilty                                                                      *
*   2. Christmas Tree Control including:                                                              *
*      a. Selectable Standard, Pro or Street Outlaw light sequencing                                  *
*      b. Staging & Foul (red-light) indicators                                                       *
*      c. Top Bulb Lock feature (enable/disable selectable)                                           *
*      d. Double Red feature  (enable/disable selectable)                                             *
*   3. Lane Sensors at the following positions:                                                       *
*      a. Pre-Stage, Stage & Guard sensors                                                            *
*      b. 60 Ft down the track sensors                                                                *
*      c. 660 Ft (1/8 Mile) sensors                                                                   *
*      d. 1/4 Mile speed trap sensors at 1254 Ft                                                      *
*      e. Finish Line sensors (1320 ft)                                                               *
*   4. Time accuracy to 3 decimal places (0.001 seconds)                                              *
*   5. Manual and Auto race start capability                                                          *
*   6. Hand-held pushbutton timer control capability                                                  *
*   7. Winner indication on corresponding Xmas Tree Green light & optional win lights at finish line  *
*   8. Sensor fault (not ready) indication & reporting                                                *
*   9. User selectable 1 or 2 racer operation                                                         *
*                                                                                                     *
* This version is designed to work in conjunction with the author's race management software          *
* DRT_RaceManager_Ver14.pde which runs on a PC attached to the Arduino MEGA via the USB port.         *
*                                                                                                     *
* This program measures the race start reaction time, down-track sensor elapsed times, and 1/4 mile   *
* speed for 2 lanes. Race status and times are sent to the serial port to interface with the race     *
* management software running on the PC. Times are measured from when the racer leaves the start line *
* to activation of the various sensors of each lane. Each racer has a preset time to finish the race. *
* If a racer doesn't cross the finish line within that time, it is assumed that the racer had a       *
* problem or the lane was not used. In either case, the value 9999.999 is assigned to the lane time   *
* as an error indication for use by the race management software. A hand-held pushbutton switch is    *
* provided to remotely start the Cristmas Tree countdown sequence or to reset the timer for the next  *
* race. This program also determines the winner IAW the NHRA rulebook.                                *
*                                                                                                     *
* Refer to the Schematic Diagram, parts list and design notes that accompany this software for more   *
* information. Feel free to contact the author at Billv923@outlook.com if any questions               *
*====================================================================================================*/

//Initialization
//Race State Definitions
#define NotReady       0              //Race State = Not Ready
#define Ready          1              //Race State = Ready
#define Staged         2              //Race State = Staged
#define Countdown      3              //Race State = Count Down Sequence in progress
#define Racing         4              //Race State = Racing
#define Finished       5              //Race State = Finished
#define Trip           1              //Sensor trip state: 0=logic 0, 1=logic 1

//Pin Assignments - Misc
#define hand_sw       10              //Manual hand-held switch - Digital Input Pin D10
//Pin Assignments - Xmas Tree Indicators
#define XtreePstg1    22              //Xmas Tree Lane1 Pre-stage Indicator - Digital Output Pin D22
#define XtreeStg1     24              //Xmas Tree Lane1 Stage Indicator - Digital Output Pin D24
#define XtreeSeq1a    26              //Xmas Tree Lane1 Amber Indicator #1 - Digital Output Pin D26
#define XtreeSeq1b    28              //Xmas Tree Lane1 Amber Indicator #2 - Digital Output Pin D28
#define XtreeSeq1c    30              //Xmas Tree Lane1 Amber Indicator #3 - Digital Output Pin D30
#define XtreeGrn1     32              //Xmas Tree Lane1 Green Go Indicator - Digital Output Pin D32
#define XtreeDsq1     34              //Xmas Tree Lane1 Disqualification Indicator - Digital Output Pin D34
#define WinLight1     36              //Lane1 Win Light - Digital Output Pin D36
#define XtreePstg2    23              //Xmas Tree Lane2 Pre-stage Indicator - Digital Output Pin D23
#define XtreeStg2     25              //Xmas Tree Lane2 stage Indicator - Digital Output Pin D25
#define XtreeSeq2a    27              //Xmas Tree Lane2 Amber Indicator #1 - Digital Output Pin D27
#define XtreeSeq2b    29              //Xmas Tree Lane2 Amber Indicator #2 - Digital Output Pin D29
#define XtreeSeq2c    31              //Xmas Tree Lane2 Amber Indicator #3 - Digital Output Pin D31
#define XtreeGrn2     33              //Xmas Tree Lane2 Green Go Indicator - Digital Output Pin D33
#define XtreeDsq2     35              //Xmas Tree Lane2 Disqualification Indicator - Digital Output Pin D35
#define WinLight2     37              //Lane2 Win Light - Digital Output Pin D37
//Pin Assignments - Lane Sensor Inputs
#define PStg_Sense1   40              //Lane1 Pre-Stage Sensor - Digital Input Pin D40
#define PStg_Sense2   41              //Lane2 Pre-Stage Sensor - Digital Input Pin D41
#define Stg_Sense1    42              //Lane1 Stage Sensor - Digital Input Pin D42
#define Stg_Sense2    43              //Lane2 Stage Sensor - Digital Input Pin D43
#define Grd_Sense1    44              //Lane1 Guard Sensor - Digital Input Pin D44
#define Grd_Sense2    45              //Lane2 Guard Sensor - Digital Input Pin D45
#define T060_Sense1   46              //Lane1 60 Foot Marker Sensor - Digital Input Pin D46
#define T060_Sense2   47              //Lane2 60 Foot Marker Sensor - Digital Input Pin D47
#define T660_Sense1   48              //Lane1 660 Foot Marker Sensor - Digital Input Pin D48
#define T660_Sense2   49              //Lane2 660 Foot Marker Sensor - Digital Input Pin D49
#define T1254_Sense1  50              //Lane1 1/4 Mile Trap Sensor - Digital Input Pin D50
#define T1254_Sense2  51              //Lane2 1/4 Mile Trap Sensor - Digital Input Pin D51
#define T1320_Sense1  52              //Lane1 1/4 Mile Finish Sensor - Digital Input Pin D52
#define T1320_Sense2  53              //Lane2 1/4 Mile Finish Sensor - Digital Input Pin D53

//Variable Initialization
boolean Hdcap_Flag;                   //True when handicap racing mode in effect
boolean Lane1_ready;                  //False if any Lane1 sensor is obstructed
boolean Lane2_ready;                  //False if any Lane2 sensor is obstructed
boolean Track_ready;                  //False if track is not ready
boolean Breakout_Flag[2];             //True if Lane breakout (E.T. < Dial-in)
boolean Stg1_Flag;                    //True if Lane1 stage sensor was tripped
boolean Stg2_Flag;                    //True if Lane2 stage sensor was tripped
boolean Grd1_Flag;                    //True if Lane1 guard sensor was tripped
boolean Grd2_Flag;                    //True if Lane2 guard sensor was tripped
boolean T060L1_Flag;                  //True if Lane1 60 Ft track sensor was tripped
boolean T060L2_Flag;                  //True if Lane2 60 Ft track sensor was tripped
boolean T660L1_Flag;                  //True if Lane1 660 Ft track sensor was tripped
boolean T660L2_Flag;                  //True if Lane2 660 Ft track sensor was tripped
boolean T1254L1_Flag;                 //True if Lane1 1/4 Mile Trap sensor was tripped
boolean T1254L2_Flag;                 //True if Lane2 1/4 Mile Trap sensor was tripped
boolean T1320L1_Flag;                 //True if Lane1 1/4 Mile Finish sensor was tripped
boolean T1320L2_Flag;                 //True if Lane2 1/4 Mile Finish sensor was tripped
boolean L1blink_Flag;                 //True if Lane1 Stage indicator is commanded to blink
boolean L2blink_Flag;                 //True if Lane2 Stage indicator is commanded to blink
boolean L1stage_Flag;                 //True if Lane1 vehicle is staged
boolean L2stage_Flag;                 //True if Lane2 vehicle is staged
boolean Hold_Flag;                    //True if 'Hold' command rec'd from PC to disable start switch
boolean Autostart_Flag;               //True if 'Auto-start' command rec'd from PC to auto start timer when staged
boolean Win_Flag[2];                  //Where true=winner
boolean WinLight_Enable;              //Set to true when optional Win Light feature is installled
boolean Grd_Enable;                   //Set to true when using guard beams
boolean TopBulbLock_Mode;             //True if "Top Bulb Lock" feature has been selected
boolean TBL_Enable;                   //Set to true for 1'st lane that does "Top Bulb Lock" so 2'nd lane acts normal
boolean DoubleRed_Flag;               //True if "Double Red" feature is active
boolean kmh_Flag = false;             //Set to true when speed is in Km/h
boolean debug = false;                //Set to true when debugging
int i,n;                              //Working variables
int Lane_Usage;                       //Lanes used: 1=Lane1 only, 2=Lane2 only, 3=Lanes 1 & 2
int Foul_Flag[2];                     //Equals 1 if 1'st to foul, 2 if 2'nd to foul
int XTmode;                           //Xmas tree sequence mode (0=Std mode, 1=Pro mode, 2=Outlaw mode)
int lane_place[2];                    //Lane finish place array
int Ln1_Seq_State;                    //Progression state of Lane1 Xmas Tree sequencing lights (0,1,2,3,4,5)
int Ln2_Seq_State;                    //Progression state of Lane2 Xmas Tree sequencing lights (0,1,2,3,4,5)
                                      //Where: 0=start, 1=Amber1, 2=Amber2, 3=Amber3, 4=Green, 5=Off
int trackstate;                       //0=NotReady, 1=Ready, 2=Staged, 3=Countdown, 4=Racing, 5=Finished
int Ln1_State;                        //Lane1: 0=NotReady, 1=Ready, 2=Staged, 3=Countdown, 4=Racing, 5=Finished
int Ln2_State;                        //Lane2: 0=NotReady, 1=Ready, 2=Staged, 3=Countdown, 4=Racing, 5=Finished
int Dialin1;                          //Lane1 dial in time (milliseconds)
int Dialin2;                          //Lane2 dial in time (milliseconds)
int Dialin_Dly1;                      //Lane1 timing start delay (milliseconds)
int Dialin_Dly2;                      //Lane2 timing start delay (milliseconds)
int ElapsedTime[2];                   //Lane elapsed time (miliseconds)
unsigned long max_duration = 20000;   //Default race duration in milliseconds (20 Sec.)
unsigned long Start_time[2];          //Time when Lane1 & Lane2 green light illuminates (milliseconds)
unsigned long Exp_Time[2];            //Lane1 & Lane2 expiration Time (milliseconds)
unsigned long Clk_Time;               //CPU Clock Time (milliseconds)
unsigned long Xtime;                  //Multi-use time variable (milliseconds)
unsigned long Seq_Time1;              //Time variable used in Lane1 Xtree Sequence routine
unsigned long Seq_Time2;              //Time variable used in Lane2 Xtree Sequence routine
unsigned long TBL_Dly;                //Time variable used in Top Bulb Osc Xtree Sequence routine
unsigned long React_time[2];          //Reaction time data array (milliseconds)
unsigned long T060_time[2];           //60 Ft Marker time data array (milliseconds)
unsigned long T660_time[2];           //660 Ft Marker time data array (milliseconds)
unsigned long T1254_time[2];          //1254 Ft Marker time data array (milliseconds)
unsigned long T1320_time[2];          //1320 Ft Marker time data array (milliseconds)
unsigned long Trp_time[2];            //1/4 Mile trap time data array (milliseconds)
unsigned long Fin_time[2];            //1/4 Mile finish time data array (milliseconds)
unsigned long blinkTime;              //Time variable used in winning lane blink routine
float Time;                           //Working variable - Calculated Lane Time (seconds)
float Temp;                           //Working temporary floating point variable
float Rtime[2];                       //Reaction time data array (seconds)
float Etime060[2];                    //60 Ft ET data array (seconds)
float Etime660[2];                    //660 Ft (1/8 Mile) ET data array (seconds)
float Etime1320[2];                   //1/4 Mile ET data array (seconds)
float Speed[2];                       //1/4 mile Speed data array (mph or kmh)
float Otime[2];                       //Race finish Overall time data array (seconds)
float Margin[2];                      //Used in handicap win determination (Elapsed time - Dial-in)
char SMsg0[4] = "NRD";                //Serial message - Not Ready
char SMsg1[4] = "RDY";                //Serial message - Ready
char SMsg2[4] = "STG";                //Serial message - Staged
char SMsg3[4] = "CNT";                //Serial message - Countdown Sequence underway
char SMsg4[4] = "RAC";                //Serial message - Racing
char SMsg5[4] = "FIN";                //Serial message - Finished
char SMsg7[4] = "TRK";                //Serial message - Track Sensor(s) blocked
char SMsg8[4] = "RST";                //Serial message - Manual RESET executed
String serial_data = "";              //Serial data input string
String error_codes = "";              //Message string populated with error codes
String Str1;                          //Working string variable

//======================================= SETUP ROUTINE =======================================
void setup() {  
  //Hardware Setup
  //Initialize Xmas Tree indicator pins as outputs; track sensors and Reset switch pins as inputs
  pinMode(XtreePstg1,OUTPUT);         //Xmas Tree Lane1 white Pre-stage Indicator
  pinMode(XtreeStg1,OUTPUT);          //Xmas Tree Lane1 white Stage Indicator
  pinMode(XtreeSeq1a,OUTPUT);         //Xmas Tree Lane1 Amber Indicator #1
  pinMode(XtreeSeq1b,OUTPUT);         //Xmas Tree Lane1 Amber Indicator #2
  pinMode(XtreeSeq1c,OUTPUT);         //Xmas Tree Lane1 Amber Indicator #3
  pinMode(XtreeGrn1,OUTPUT);          //Xmas Tree Lane1 Green Go Indicator
  pinMode(XtreeDsq1,OUTPUT);          //Xmas Tree Lane1 Disqualification Indicator
  pinMode(XtreePstg2,OUTPUT);         //Xmas Tree Lane2 white Pre-stage Indicator
  pinMode(XtreeStg2,OUTPUT);          //Xmas Tree Lane2 white Stage Indicator
  pinMode(XtreeSeq2a,OUTPUT);         //Xmas Tree Lane2 Amber Indicator #1
  pinMode(XtreeSeq2b,OUTPUT);         //Xmas Tree Lane2 Amber Indicator #2
  pinMode(XtreeSeq2c,OUTPUT);         //Xmas Tree Lane2 Amber Indicator #3
  pinMode(XtreeGrn2,OUTPUT);          //Xmas Tree Lane2 Green Go Indicator
  pinMode(XtreeDsq2,OUTPUT);          //Xmas Tree Lane2 Disqualification Indicator
  pinMode(WinLight1,OUTPUT);          //Lane1 Win Indicator
  pinMode(WinLight2,OUTPUT);          //Lane2 Win Indicator
  
  //Note: Sensor inputs transition from LOW to HIGH when light beam is interrupted
  pinMode(PStg_Sense1,INPUT_PULLUP);  //Lane1 Pre-tage sensor
  pinMode(PStg_Sense2,INPUT_PULLUP);  //Lane2 Pre-tage sensor
  pinMode(Stg_Sense1,INPUT_PULLUP);   //Lane1 Stage sensor
  pinMode(Stg_Sense2,INPUT_PULLUP);   //Lane2 Stage sensor
  pinMode(Grd_Sense1,INPUT_PULLUP);   //Lane1 Guard sensor
  pinMode(Grd_Sense2,INPUT_PULLUP);   //Lane2 Guard sensor
  pinMode(T060_Sense1,INPUT_PULLUP);  //Lane1 60 ft track sensor
  pinMode(T060_Sense2,INPUT_PULLUP);  //Lane2 60 ft track sensor
  pinMode(T660_Sense1,INPUT_PULLUP);  //Lane1 660 ft track sensor
  pinMode(T660_Sense2,INPUT_PULLUP);  //Lane2 660 ft track sensor
  pinMode(T1254_Sense1,INPUT_PULLUP); //Lane1 1/4 Mile Trap sensor
  pinMode(T1254_Sense2,INPUT_PULLUP); //Lane2 1/4 Mile Trap sensor
  pinMode(T1320_Sense1,INPUT_PULLUP); //Lane1 1/4 Mile Finish line sensor
  pinMode(T1320_Sense2,INPUT_PULLUP); //Lane2 1/4 Mile Finish line sensor
  pinMode(hand_sw,INPUT_PULLUP);      //Switch is normally HIGH & transitions to LOW when pressed
  
  //Software Setup
  Turnoff_All_Lights();               //Call subroutine to turn off Xmas Tree lights
  Serial.begin(9600);                 //Enable serial comms to PC at 9600 baud on Port0
  trackstate = NotReady;              //Set initial track state to not ready
  Ln1_State = NotReady;               //Set Lane1 initial race state to not ready
  Ln2_State = NotReady;               //Set Lane2 initial race state to not ready
  Foul_Flag[0] = 0;                   //Preset Lane1 foul flag to 0
  Foul_Flag[1] = 0;                   //Preset Lane2 foul flag to 0
  XTmode = 0;                         //Set Xmas tree sequence to Std mode
  Autostart_Flag = false;             //Preset Autostart flag to "manual" start
  TopBulbLock_Mode = false;           //Preset Top Bulb Lock feature to disabled
  TBL_Enable = false;                 //Preset Top Bulb Lock to disabled
  DoubleRed_Flag = false;             //Preset Double Red feature to disabled
  Lane_Usage = 3;                     //Preset for 2-racer mode
  Grd_Enable = true;                  //Enable Guard Beam capability if they're used
  WinLight_Enable = false;            //Finish line Win Lights: true=Enabled, false=disabled
}

//======================================== MAIN LOOP ============================================
void loop() {
  //CHECK & PROCESS INCOMING SERIAL DATA
  if (Serial.available() > 0) {                     //Check if serial data
    serial_data = Serial.readStringUntil('\n');     //And get it if so
    Serial.flush();
    if (serial_data.indexOf("RS") >= 0) {           //Check for Reset request from PC
      timer_reset();                                //Call subroutine to reset the timer
    }
    if (serial_data.indexOf("ST") >= 0) {           //Check for race start request from PC
      if (trackstate == Staged) {                   //Check if vehicles are staged
        Set_Dialin_Delays();                        //Call subroutine to set dial-in delays
        trackstate = Countdown;                     //Set race state to Countdown mode
        if (Lane_Usage==1 || Lane_Usage==3) Ln1_State = Countdown; //Set Lane1 state to Countdown state if used
        if (Lane_Usage==2 || Lane_Usage==3) Ln2_State = Countdown; //Set Lane2 state to Countdown state if used
        Serial.println(SMsg3);                      //Send Countdown state message to PC
      }
    }
    if (serial_data.indexOf('H') >= 0) {            //Check for hold request from PC
      Hold_Flag = true;                             //Set Hold flag
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf('C') >= 0) {            //Check for continue request from PC
      Hold_Flag = false;                            //Clear Hold flag
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf("LT") >= 0) {           //Check for Xmas tree lamp test request from PC
      Lamptest();                                   //Call Xmas Tree Lamp Test subroutine
      if (debug) Serial.println(serial_data);       //For debug only
    }
    if (serial_data.indexOf("M:") >= 0) {           //Check for Xmas Tree mode request from PC
      Str1 = serial_data.substring(2, 3);           //Extract Xmas Tree mode code
      if (Str1=="0") XTmode = 0;                    //Set Xmas Tree to Std Mode if true
      if (Str1=="1") XTmode = 1;                    //Set Xmas Tree to Pro Mode if true
      if (Str1=="2") XTmode = 2;                    //Set Xmas Tree to Outlaw Mode if true
      if (debug) Serial.println(serial_data);       //For debug only
    }
    if (serial_data.indexOf("AS:") >= 0) {          //Check for timer auto-start request from PC
      Str1 = serial_data.substring(3, 4);           //Extract auto-start enable/disable code
      if (Str1=="1") Autostart_Flag = true;         //Set timer to Auto-start mode
      else Autostart_Flag = false;                  //Set timer to Manual-start mode
      if (debug) Serial.println(serial_data);       //For debug only
    }
    if (serial_data.indexOf("LN:") >= 0) {          //Check for lane usage request from PC
      Str1 = serial_data.substring(3, 4);           //Extract lane usage code
      if (Str1=="1") Lane_Usage = 1;                //Set to single racer on Lane 1 mode
      if (Str1=="2") Lane_Usage = 2;                //Set to single racer on Lane 2 mode
      if (Str1=="3") Lane_Usage = 3;                //Set to dual racer mode
      if (debug) Serial.println(serial_data);       //For debug only
    }
    if (serial_data.indexOf("KMH:") >= 0) {         //Check for Metric speed enable/disable request from PC
      Str1 = serial_data.substring(4, 5);           //Extract enable/disable code
      if (Str1=="1") kmh_Flag = true;               //Set metric speed flag to true if "1"
      else kmh_Flag = false;                        //Set metric speed flag to false if not "1"
      if (debug) Serial.println(serial_data);       //For debug only
    }
    if (serial_data.indexOf("TBL:") >= 0) {         //Check for "Top Bulb Lock" enable/disable request from PC
      Str1 = serial_data.substring(4, 5);           //Extract enable/disable code
      if (Str1=="1") {
        TopBulbLock_Mode = true;                    //Set Top Bulb Lock mode to true if "1"
        TBL_Enable = true;                          //Enable Top Bulb Lock
      }
      else {
        TopBulbLock_Mode = false;                   //Set Top Bulb Lock mode to false if not "1"
        TBL_Enable = false;                         //Disable Top Bulb Lock
      }
      if (debug) Serial.println(serial_data);       //For debug only
    }
    if (serial_data.indexOf("DBR:") >= 0) {         //Check for "Double Red" enable/disable request from PC
      Str1 = serial_data.substring(4, 5);           //Extract enable/disable code
      if (Str1=="1") DoubleRed_Flag = true;         //Set Double Red flag to true if "1"
      else DoubleRed_Flag = false;                  //Set Double Red flag to false if not "1"
      if (debug) Serial.println(serial_data);       //For debug only
    }
    if (serial_data.indexOf("D1:") >= 0) {          //Check for receipt of Lane1 dial-in time from PC
      //Input string format: D1:xxxxx where xxxxx is Lane1 dial-in time in milliseconds
      //Valid range: 00000 - 32767 (0.0 to 32.767 seconds)
      if (serial_data.length() >= 8) {              //Check integrity of input string
        Str1 = serial_data.substring(3, 8);         //Extract Lane1 dial-in time
        Dialin1 = Str1.toInt();                     //And convert to integer
        if (debug) Serial.println(Dialin1);         //For debug only
      }
    }
    if (serial_data.indexOf("D2:") >= 0) {          //Check for receipt of Lane2 dial-in time from PC
      //Input string format: D2:xxxxx where xxxxx is Lane2 dial-in time in milliseconds
      //Valid range: 00000 - 32767 (0.0 to 32.767 seconds)
      if (serial_data.length() >= 8) {              //Check integrity of input string
        Str1 = serial_data.substring(3, 8);         //Extract Lane2 dial-in time
        Dialin2 = Str1.toInt();                     //And convert to integer
        if (debug) Serial.println(Dialin2);         //For debug only
      }
    }
    if (serial_data.indexOf("TO:") >= 0) {          //Check for receipt of timer timeout duration from PC
      //Input string format: TO:xx where xx is the timeout duration in seconds
      //Valid range: 05 - 30
      if (serial_data.length() >= 5) {              //Check integrity of input string
        Str1 = serial_data.substring(3, 5);         //Extract timeout duration in seconds
        max_duration = Str1.toInt() * 1000;         //And convert to milliseconds
        if (debug) Serial.println(max_duration);    //For debug only
      }
    }
    if (serial_data.indexOf("DBug") >= 0) {         //Check for receipt of debug request from PC
      debug = !debug;                               //Toggle debug flag
      if (debug) Serial.println("Debug=True");      //And send debug state to PC
      else Serial.println("Debug=False");
    }
    serial_data = "";
  }

  //CHECK & PROCESS LANE 1 STAGING SENSORS
  if (digitalRead(PStg_Sense1)==Trip) {             //Is Lane1 Pre-Stage sensor tripped?
    digitalWrite(XtreePstg1,LOW);                   //Turn on Lane1 Pre-Stage Indicator
  }
  else digitalWrite(XtreePstg1,HIGH);               //Turn off Lane1 Pre-Stage Indicator
  if (digitalRead(Stg_Sense1)==Trip) {              //Is Lane1 Stage sensor tripped?
    digitalWrite(XtreeStg1,LOW);                    //Turn on Lane1 Stage Indicator
    Stg1_Flag = true;                               //Set Lane1 as staged
  }
  else {                                            //Lane1 Stage sensor not tripped
    digitalWrite(XtreeStg1,HIGH);                   //Turn off Lane1 Stage Indicator
    Stg1_Flag = false;                              //Set Lane1 as not staged
  }
  //CHECK & PROCESS LANE 2 STAGING SENSORS
  if (digitalRead(PStg_Sense2)==Trip) {             //Is Lane2 Pre-Stage sensor tripped?
    digitalWrite(XtreePstg2,LOW);                   //Turn on Lane2 Pre-Stage Indicator
  }
  else digitalWrite(XtreePstg2,HIGH);               //Turn off Lane2 Pre-Stage Indicator
  if (digitalRead(Stg_Sense2)==Trip) {              //Is Lane2 Stage sensor tripped?
    digitalWrite(XtreeStg2,LOW);                    //Turn on Lane2 Stage Indicator
    Stg2_Flag = true;                               //Set Lane2 as staged
  }
  else {                                            //Lane2 Stage sensor not tripped
    digitalWrite(XtreeStg2,HIGH);                   //Turn off Lane2 Stage Indicator
    Stg2_Flag = false;                              //Set Lane2 as not staged
  }

  //CHECK & PROCESS LANE 1 & 2 GAURD SENSORS IF THEY'RE USED
  if (Grd_Enable) {                                 //Do if Guard Sensor feature enabled?
    if (digitalRead(Grd_Sense1)==Trip) {            //Is Lane1 Guard sensor tripped?
      Grd1_Flag = true;                             //Set Lane1 Guard as tripped
    }
    else Grd1_Flag = false;                         //Set Lane1 Guard as not tripped
    if (digitalRead(Grd_Sense2)==Trip) {            //Is Lane2 Guard sensor tripped?
      Grd2_Flag = true;                             //Set Lane2 Guard as tripped
    }
    else Grd2_Flag = false;                         //Set Lane2 Guard as not tripped
    if (trackstate==Ready || trackstate == Staged) {
      if (Grd1_Flag) digitalWrite(XtreeDsq1,LOW);   //Turn on Lane1 red foul light
        else digitalWrite(XtreeDsq1,HIGH);          //Turn off Lane1 red foul light
      if (Grd2_Flag) digitalWrite(XtreeDsq2,LOW);   //Turn on Lane2 red foul light
        else digitalWrite(XtreeDsq2,HIGH);          //Turn off Lane2 red foul light
    }
  }
 
  //READY STATE
  if (trackstate == Ready) {                        //Do if track in Ready race state
    if (Lane_Usage==1) {                            //Do if only 1 car racing on Lane1
      if (Stg1_Flag && !Grd1_Flag) {                //Is Lane1 vehicle staged?
        delay(250);                                 //Wait 1/4 second
        if (digitalRead(Stg_Sense1)==Trip) {        //Is Lane1 vehicle still staged
          Ln1_State = Staged;                       //Set Lane1 race state to Staged if so
          trackstate = Staged;                      //Set track race state to Staged if so
          L1stage_Flag = true;                      //Set the Lane1 staged flag
          Serial.println(SMsg2);                    //Send Staged state message to PC
        }
      }
    }
    if (Lane_Usage==2) {                            //Do if only 1 car racing on Lane2
      if (Stg2_Flag && !Grd2_Flag) {                //Is Lane2 vehicle staged?
        delay(250);                                 //Wait 1/4 second
        if (digitalRead(Stg_Sense2)==Trip) {        //Is Lane2 vehicle still staged
          Ln2_State = Staged;                       //Set Lane2 race state to Staged if so
          trackstate = Staged;                      //Set track race state to Staged if so
          L2stage_Flag = true;                      //Set the Lane2 staged flag
          Serial.println(SMsg2);                    //Send Staged state message to PC
        }
      }
    }
    if (Lane_Usage==3) {                            //Do if 2 cars are racing
      if (Stg1_Flag && !Grd1_Flag && Stg2_Flag && !Grd2_Flag) { //Are both vehicles staged?
        delay(250);                                 //Wait 1/4 second
        if (digitalRead(Stg_Sense1)==Trip && digitalRead(Stg_Sense2)==Trip) { //Are both vehicles still staged?
          Ln1_State = Staged;                       //Set Lane1 race state to Staged if so
          Ln2_State = Staged;                       //Set Lane2 race state to Staged if so
          trackstate = Staged;                      //Set track race state to Staged if so
          L1stage_Flag = true;                      //Set the Lane1 staged flag
          L2stage_Flag = true;                      //Set the Lane2 staged flag
          Serial.println(SMsg2);                    //Send Staged state message to PC
        }
      }
    }
  }

  //STAGED STATE
  if (trackstate == Staged) {                       //Do if track is in the Staged state
    if (!Hold_Flag) {                               //Disable race start if Hold flag is set
      if (Autostart_Flag) {                         //Do auto start if flag is set
        delay(random(500, 1500));                   //Random wait between 0.5 and 1.5 sec before we start countdown
        Set_Dialin_Delays();                        //Call subroutine to set dial-in delays
        trackstate = Countdown;                     //Set track state to Countdown state
        if (Lane_Usage==1 || Lane_Usage==3) Ln1_State = Countdown; //Set Lane1 state to Countdown state if used
        if (Lane_Usage==2 || Lane_Usage==3) Ln2_State = Countdown; //Set Lane2 state to Countdown state if used
        Serial.println(SMsg3);                      //Send Countdown state message to PC
      }
      else {                                        //Do manual start if Autostart_Flag is not set
        while(digitalRead(hand_sw)==0) {            //Do while hand switch is pressed
          if (millis() > (Clk_Time + 500)) {        //Has switch been held for 1/2 second?
            Set_Dialin_Delays();                    //Call subroutine to set dial-in delays
            trackstate = Countdown;                 //Set track state to Countdown
            if (Lane_Usage==1 || Lane_Usage==3) Ln1_State = Countdown; //Set Lane1 state to Countdown state if used
            if (Lane_Usage==2 || Lane_Usage==3) Ln2_State = Countdown; //Set Lane2 state to Countdown state if used
            Serial.println(SMsg3);                  //Send Countdown state message to PC
            break;                                  //exit while loop
          }
          delay(50);                                //Wait awhile before we loop again
        }
      }
    }
    //The following resets timer to ready state if a car un-stages before the countdown begins
    if (Lane_Usage==1) {                            //Do if 1 racer on Lane1
      if (!Stg1_Flag || Grd1_Flag) {                //Is Lane1 vehicle no longer staged?
        trackstate = Ready;                         //Revert back to Ready state
        Serial.println(SMsg1);                      //Send Ready state message to PC
      }
    }
    if (Lane_Usage==2) {                            //Do if 1 racer on Lane2
      if (!Stg2_Flag || Grd2_Flag) {                //Is Lane2 vehicle no longer staged?
        trackstate = Ready;                         //Revert back to Ready state
        Serial.println(SMsg1);                      //Send Ready state message to PC
      }
    }
    if (Lane_Usage==3) {                            //Do if 2 racers
      if (!Stg1_Flag || Grd1_Flag || !Stg2_Flag || Grd2_Flag) { //Is either Lane1 or Lane2 no longer staged?
        trackstate = Ready;                         //Revert back to Ready state
        Serial.println(SMsg1);                      //Send Ready state message to PC
      }
    }
  }
  
  //NOTE: Clock time is used extensively when the timer is in the countdown & racing states.
  //      So for speed & efficiency we get the current time just once each pass through the loop
  Clk_Time = millis();                              //Get CPU clock time
  
  //LANE 1 COUNTDOWN SEQUENCE
  if (Ln1_State == Countdown) {                     //Do if Lane1 in Countdown state
    if (XTmode == 0) {                              //Do standard Xmas Tree lamp sequence
      switch (Ln1_Seq_State) {                      //Get Lane1 sequence state
        case 0:
          Seq_Time1 = Clk_Time + Dialin_Dly1;       //Set dial-in delay for next sequence
          Ln1_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on first amber light
          if (Clk_Time > (Seq_Time1+TBL_Dly)) {     //Check if time to turn on Amber1 light
            if (Foul_Flag[0]!=1) {                  //Do if not fouled
              digitalWrite(XtreeSeq1a,LOW);         //Turn on Lane1 Amber Indicator #1
              if (TBL_Enable && Hdcap_Flag) {       //Do if Top Bulb Lock enabled, Handicapped race & 1'st lane to light Amber#1
                if (Foul_Flag[1]!=1)digitalWrite(XtreeSeq2a,LOW); //Turn on Lane2 Amber Indicator #1 if not fouled
                TBL_Enable = false;                 //Disable Top Bulb Lock
              }
            }
            Seq_Time1 = Seq_Time1 + 500;            //Add another 0.5 sec delay for next sequence
            Ln1_Seq_State = 2;                      //Set Sequence State count to 2 (Amber2 on)
          }
          break;
        case 2:                                     //Turn on second amber light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Amber2 light
            digitalWrite(XtreeSeq1a,HIGH);          //Turn off Lane1 Amber Indicator #1
            if (Foul_Flag[0]!=1) digitalWrite(XtreeSeq1b,LOW); //Turn on Lane1 Amber Indicator #2 if not fouled
            Seq_Time1 = Seq_Time1 + 500;            //Add another 0.5 sec delay for next sequence
            Ln1_Seq_State = 3;                      //Set Sequence State count to 3 (Amber3 on)
          }
          break;
        case 3:                                     //Turn on third amber light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Amber3 light
            digitalWrite(XtreeSeq1b,HIGH);          //Turn off Lane1 Amber Indicator #2
            if (Foul_Flag[0]!=1) digitalWrite(XtreeSeq1c,LOW); //Turn on Lane1 Amber Indicator #3 if not fouled
            Seq_Time1 = Seq_Time1 + 500;            //Add another 0.5 sec delay for next sequence
            Ln1_Seq_State = 4;                      //Set Sequence State count to 4 (Green on)
          }
          break;
        case 4:                                     //Turn on green light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq1c,HIGH);          //Turn off Lane1 Amber Indicator #3
            if (Foul_Flag[0]!=1) digitalWrite(XtreeGrn1,LOW); //Turn on Lane1 Green Go Indicator if not fouled
            Ln1_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln1_State = Racing;                     //Set Lane1 race state to Racing
            Start_time[0] = Clk_Time;               //Set Lane1 race start time
            Exp_Time[0] = Clk_Time + max_duration;  //Set Lane1 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 1) {                              //Do Pro Xmas Tree lamp sequence
      switch (Ln1_Seq_State) {                      //Get Lane1 sequence state
        case 0:
          Seq_Time1 = Clk_Time + Dialin_Dly1;       //Set dial in delay for next sequence
          Ln1_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on all amber lights
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Amber lights
            if (Foul_Flag[0]!=1) {                  //Do if Lane1 hasn't fouled
              digitalWrite(XtreeSeq1a,LOW);         //Turn on Lane1 Amber Indicator #1
              digitalWrite(XtreeSeq1b,LOW);         //Turn on Lane1 Amber Indicator #2
              digitalWrite(XtreeSeq1c,LOW);         //Turn on Lane1 Amber Indicator #3
            }
            Seq_Time1 = Seq_Time1 + 400;            //Add another 400 ms delay for next sequence
            Ln1_Seq_State = 2;                      //Set Sequence State count to 2
          }
          break;
        case 2:                                     //Turn off amber lights & turn on green light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq1a,HIGH);          //Turn off Lane1 Amber Indicator #1
            digitalWrite(XtreeSeq1b,HIGH);          //Turn off Lane1 Amber Indicator #2
            digitalWrite(XtreeSeq1c,HIGH);          //Turn off Lane1 Amber Indicator #3
            if (Foul_Flag[0]!=1) digitalWrite(XtreeGrn1,LOW); //Turn on Lane1 Green Go Indicator if not fouled
            Ln1_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln1_State = Racing;                     //Set Lane1 race state to Racing
            Start_time[0] = Clk_Time;               //Set Lane1 race start time
            Exp_Time[0] = Clk_Time + max_duration;  //Set Lane1 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 2) {                              //Do Outlaw Xmas Tree lamp sequence
      switch (Ln1_Seq_State) {                      //Get Lane1 sequence state
        case 0:
          Seq_Time1 = Clk_Time + Dialin_Dly1;       //Set dial in delay for next sequence
          Ln1_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Green light
            if (Foul_Flag[0]!=1) digitalWrite(XtreeGrn1,LOW); //Turn on Lane1 Green Go Indicator if not fouled
            Ln1_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln1_State = Racing;                     //Set Lane1 race state to Racing
            Start_time[0] = Clk_Time;               //Set Lane1 race start time
            Exp_Time[0] = Clk_Time + max_duration;  //Set Lane1 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (Foul_Flag[0]==0 && (!Stg1_Flag || Grd1_Flag)) { //Did Lane1 vehicle leave early during countdown?
      React_time[0] = Clk_Time;                     //Get Lane1 reaction time
      Foul_Flag[0] = Foul_Flag[1] + 1;              //Set Lane1 foul flag as 1'st or 2'nd to foul
      if (Foul_Flag[0]==1 || DoubleRed_Flag) digitalWrite(XtreeDsq1,LOW); //red-light Lane1 if fouled 1'st or DoubleRed_Flag set
    }
  }
  //LANE 2 COUNTDOWN SEQUENCE
  if (Ln2_State==Countdown) {                       //Do if Lane2 in Countdown state
    if (XTmode == 0) {                              //Do standard Xmas Tree lamp sequence
      switch (Ln2_Seq_State) {                      //Get Lane2 sequence state
        case 0:
          Seq_Time2 = Clk_Time + Dialin_Dly2;       //Set dial-in delay for next sequence
          Ln2_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on first amber light
          if (Clk_Time > (Seq_Time2+TBL_Dly)) {     //Check if time to turn on Amber1 light
            if (Foul_Flag[1]!=1) {
              digitalWrite(XtreeSeq2a,LOW);         //Turn on Lane2 Amber Indicator #1 if not fouled
              if (TBL_Enable && Hdcap_Flag) {       //Do if Top Bulb Lock enabled, Handicapped race & 1'st lane to light Amber#1
                if (Foul_Flag[0]!=1)digitalWrite(XtreeSeq1a,LOW); //Turn on Lane1 Amber Indicator #1 if not fouled
                TBL_Enable = false;                 //Disable Top Bulb Lock
              }
            }
            Seq_Time2 = Seq_Time2 + 500;            //Add another 0.5 sec delay for next sequence
            Ln2_Seq_State = 2;                      //Set Sequence State count to 2 (Amber2 on)
          }
          break;
        case 2:                                     //Turn on second amber light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Amber2 light
            digitalWrite(XtreeSeq2a,HIGH);          //Turn off Lane2 Amber Indicator #1
            if (Foul_Flag[1]!=1) digitalWrite(XtreeSeq2b,LOW); //Turn on Lane2 Amber Indicator #2 if not fouled
            Seq_Time2 = Seq_Time2 + 500;            //Add another 0.5 sec delay for next sequence
            Ln2_Seq_State = 3;                      //Set Sequence State count to 3 (Amber3 on)
          }
          break;
        case 3:                                     //Turn on third amber light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Amber3 light
            digitalWrite(XtreeSeq2b,HIGH);          //Turn off Lane2 Amber Indicator #2
            if (Foul_Flag[1]!=1) digitalWrite(XtreeSeq2c,LOW); //Turn on Lane2 Amber Indicator #3 if not fouled
            Seq_Time2 = Seq_Time2 + 500;            //Add another 0.5 sec delay for next sequence
            Ln2_Seq_State = 4;                      //Set Sequence State count to 4 (Green on)
          }
          break;
        case 4:                                     //Turn on green light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq2c,HIGH);          //Turn off Lane2 Amber Indicator #3
            if (Foul_Flag[1]!=1) digitalWrite(XtreeGrn2,LOW); //Turn on Lane2 Green Go Indicator if not fouled
            Ln2_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln2_State = Racing;                     //Set Lane2 race state to Racing
            Start_time[1] = Clk_Time;               //Set Lane2 race start time
            Exp_Time[1] = Clk_Time + max_duration;  //Set Lane2 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 1) {                              //Do Pro Xmas Tree lamp sequence
      switch (Ln2_Seq_State) {
        case 0:
          Seq_Time2 = Clk_Time + Dialin_Dly2;       //Set dial in delay for next sequence
          Ln2_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on all amber lights
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Amber lights
            if (Foul_Flag[1]!=1) {                  //Do if Lane2 hasn't fouled
              digitalWrite(XtreeSeq2a,LOW);         //Turn on Lane2 Amber Indicator #1
              digitalWrite(XtreeSeq2b,LOW);         //Turn on Lane2 Amber Indicator #2
              digitalWrite(XtreeSeq2c,LOW);         //Turn on Lane2 Amber Indicator #3
            }
            Seq_Time2 = Seq_Time2 + 400;            //Add another 400 ms delay for next sequence
            Ln2_Seq_State = 2;                      //Set Sequence State count to 2 (Amber2 on)
          }
          break;
        case 2:                                     //Turn off amber lights & turn on green light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq2a,HIGH);          //Turn off Lane2 Amber Indicator #1
            digitalWrite(XtreeSeq2b,HIGH);          //Turn off Lane2 Amber Indicator #2
            digitalWrite(XtreeSeq2c,HIGH);          //Turn off Lane2 Amber Indicator #3
            if (Foul_Flag[1]!=1) digitalWrite(XtreeGrn2,LOW); //Turn on Lane2 Green Go Indicator if not fouled
            Ln2_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln2_State = Racing;                     //Set Lane2 race state to Racing
            Start_time[1] = Clk_Time;               //Set Lane2 race start time
            Exp_Time[1] = Clk_Time + max_duration;  //Set Lane2 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 2) {                              //Do Outlaw Xmas Tree lamp sequence
      switch (Ln2_Seq_State) {
        case 0:
          Seq_Time2 = Clk_Time + Dialin_Dly2;       //Set dial in delay for next sequence
          Ln2_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Green light
            if (Foul_Flag[1]!=1) digitalWrite(XtreeGrn2,LOW); //Turn on Lane2 Green Go Indicator if not fouled
            Ln2_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln2_State = Racing;                     //Set Lane2 race state to Racing
            Start_time[1] = Clk_Time;               //Set Lane2 race start time
            Exp_Time[1] = Clk_Time + max_duration;  //Set Lane2 expiration time till timeout
          }
        default:
          //Do nothing
          break;
      }
    }
    if (Foul_Flag[1]==0 && (!Stg2_Flag || Grd2_Flag)) { //Did Lane2 vehicle leave early during countdown?
      React_time[1] = Clk_Time;                     //Get Lane2 reaction time
      Foul_Flag[1] = Foul_Flag[0] + 1;              //Set Lane2 foul flag as 1'st or 2'nd to foul
      if (Foul_Flag[1]==1 || DoubleRed_Flag) digitalWrite(XtreeDsq2,LOW); //red-light Lane1 if fouled 1'st or DoubleRed_Flag set
    }
  }

  //LANE 1 RACING STATE
  if (Ln1_State == Racing) {                        //Lane1 race underway. Check/Wait for Lane1 to finish
    if (L1stage_Flag && Foul_Flag[0]==0) {          //Check & skip if Lane1 already left
      if (!Stg1_Flag || Grd1_Flag) {                //Did Lane1 vehicle leave start line?
        React_time[0] = Clk_Time;                   //Yes...get time
        L1stage_Flag = false;                       //Set Lane1 as not staged
      }
    }
    if (!T060L1_Flag) {                             //Check & skip if Lane1 already tripped 60 Ft sensor
      if (digitalRead(T060_Sense1)==Trip) {         //Is Lane1 60 Ft sensor tripped?
        T060_time[0] = Clk_Time;                    //Yes...get time
        T060L1_Flag = true;                         //Set Lane1 60 Ft sensed flag
      }
    }
    if (!T660L1_Flag) {                             //Check & skip if Lane1 already tripped 660 Ft sensor
      if (digitalRead(T660_Sense1)==Trip) {         //Is Lane1 660 Ft sensor tripped?
        T660_time[0] = Clk_Time;                    //Yes...get time
        T660L1_Flag = true;                         //Set Lane1 660 Ft sensed flag
      }
    }
    if (!T1254L1_Flag) {                            //Check & skip if Lane1 already tripped 1/4 Mile Trap sensor
      if (digitalRead(T1254_Sense1)==Trip) {        //Is Lane1 1/4 Mile trap sensor tripped?
        T1254_time[0] = Clk_Time;                   //Yes...get time
        T1254L1_Flag = true;                        //Set Lane1 1/4 Mile trap sensed flag
      }
    }
    if (!T1320L1_Flag) {                            //Check & skip if Lane1 already crossed 1/4 Mile finish line
      if (digitalRead(T1320_Sense1)==Trip) {        //Is Car over Lane1 1/4 Mile Finish Line?
        T1320_time[0] = Clk_Time;                   //Yes...get time
        Ln1_State = Finished;                       //Set Lane1 state to finished
        T1320L1_Flag = true;                        //Set Lane1 1/4 Mile finished sensed flag
      }
    }
    if (Clk_Time>Exp_Time[0]) {                     //Has Lane1 race time expired
      Ln1_State = Finished;                         //Set Lane1 state to finished
    }
  }
  //LANE 2 RACING STATE
  if (Ln2_State == Racing) {                        //Lane2 race underway. Check/Wait for Lane2 to finish
    if (L2stage_Flag && Foul_Flag[1]==0) {          //Check & skip if Lane2 car already left
      if (!Stg2_Flag || Grd2_Flag) {                //Did Lane2 vehicle leave start line?
        React_time[1] = Clk_Time;                   //Yes...get time
        L2stage_Flag = false;                       //Set Lane2 as not staged
      }
    }
    if (!T060L2_Flag) {                             //Check & skip if Lane2 already tripped 60 Ft sensor
      if (digitalRead(T060_Sense2)==Trip) {         //Is Lane2 60 Ft sensor tripped?
        T060_time[1] = Clk_Time;                    //Yes...get time
        T060L2_Flag = true;                         //Set Lane2 60 Ft sensed flag
      }
    }
    if (!T660L2_Flag) {                             //Check & skip if Lane2 already tripped 660 Ft sensor
      if (digitalRead(T660_Sense2)==Trip) {         //Is Lane2 660 Ft sensor tripped?
        T660_time[1] = Clk_Time;                    //Yes...get time
        T660L2_Flag = true;                         //Set Lane2 660 Ft sensed flag
      }
    }
    if (!T1254L2_Flag) {                            //Check & skip if Lane2 already tripped 1/4 Mile Trap sensor
      if (digitalRead(T1254_Sense2)==Trip) {        //Is Lane2 1/4 Mile trap sensor tripped?
        T1254_time[1] = Clk_Time;                   //Yes...get time
        T1254L2_Flag = true;                        //Set Lane2 1/4 Mile trap sensed flag
      }
    }
    if (!T1320L2_Flag) {                            //Check & skip if Lane2 already crossed 1/4 Mile finish line
      if (digitalRead(T1320_Sense2)==Trip) {        //Is Car over 1/4 Mile Lane2 Finish Line?
        T1320_time[1] = Clk_Time;                   //Yes...get time
        Ln2_State = Finished;                       //Set Lane2 state to finished
        T1320L2_Flag = true;                        //Set Lane2 1/4 Mile finished sensed flag
      }
    }
    if (Clk_Time>Exp_Time[1]) {                     //Has Lane2 race time expired
      Ln2_State = Finished;                         //Set Lane2 state to finished
    }
  }

  //CHECK FOR RACE IN PROGRESS
  if (trackstate == Countdown) {                    //Do only if in coundown state
    if (Lane_Usage!=3) {                            //If one racer
      if (Ln1_State==Racing || Ln2_State==Racing) { //Is either cars in racing state?
        trackstate = Racing;                        //Set track state to Racing
        Serial.println(SMsg4);                      //Send Racing state message to PC
      }
    }
    else {                                          //If two racers
      if (Ln1_State==Racing && Ln2_State==Racing) { //Are both cars in racing state?
        trackstate = Racing;                        //Set track state to Racing
        Serial.println(SMsg4);                      //Send Racing state message to PC
      }
    }
  }
  
  //CHECK FOR RACE COMPLETE
  if (trackstate == Racing) {                       //Do only if in Racing state
    if (Lane_Usage==1 && Ln1_State==Finished) {     //If one racer in Lane1
      trackstate = Finished;                        //Set racestate to Finished if Lane1 finished
      digitalWrite(XtreeGrn1,HIGH);                 //Turn off Lane1 Green Go Indicator
    }
    if (Lane_Usage==2 && Ln2_State==Finished) {     //If one racer in Lane2
      trackstate = Finished;                        //Set racestate to Finished if Lane2 finished
      digitalWrite(XtreeGrn2,HIGH);                 //Turn off Lane2 Green Go Indicator
    }
    if (Lane_Usage==3) {                            //If two racers
      if (Ln1_State==Finished && Ln2_State==Finished) {
        trackstate = Finished;                      //set racestate to Finished if both finished
        digitalWrite(XtreeGrn1,HIGH);               //Turn off Lane1 Green Go Indicator
        digitalWrite(XtreeGrn2,HIGH);               //Turn off Lane2 Green Go Indicator
      }
    }
  }

  //RACE IS OVER OR TIME HAS EXPIRED
  if (trackstate==Finished) {                       //Is race complete?
    //Calculate times & speed
    if (Lane_Usage==1 || Lane_Usage==3) {           //Do if Lane1 was used
      Calculate_Times(0);                           //Call subroutine to calculate times for Lane1
    }
    if (Lane_Usage==2 || Lane_Usage==3) {           //Do if Lane2 was used
      Calculate_Times(1);                           //Call subroutine to calculate times for Lane2
    }
    //Determine winner for non-handicap race
    if (!Hdcap_Flag) {                              //Do if not a Handicap race
      if (Lane_Usage==3) {                          //Do if 2 vehicles racing
        if (Etime1320[0]>0.0 && Etime1320[1]>0.0) { //If both lanes finished
          if (Foul_Flag[0]==0 && Foul_Flag[1]==0) { //If both lanes didn't foul
            if (Otime[0]<Otime[1]) Win_Flag[0]=true; //Lane1 is winner if Lane1 finished first
            if (Otime[0]>Otime[1]) Win_Flag[1]=true; //Lane2 is winner if Lane2 finished first
            if (Otime[0]==Otime[1]) {                //If both lanes tied
              //DRAW: Use speed to determine winner
              if (Speed[0]>Speed[1]) Win_Flag[0]=true; //Lane1 is winner
              if (Speed[1]>Speed[0]) Win_Flag[1]=true; //Lane2 is winner
            }
          }
          if (Foul_Flag[0]==1 && Foul_Flag[1]==0) { //If only Lane1 fouled
            Win_Flag[1]=true;                       //Lane2 is winner
          }
          if (Foul_Flag[0]==0 && Foul_Flag[1]==1) { //If only Lane2 fouled
            Win_Flag[0]=true;                       //Lane1 is winner
          }
          if (Foul_Flag[0]>0 && Foul_Flag[1]>0) {   //If both lanes fouled
            //The one who fouled first is the loser. (i.e. One with shortest negative React time is winner?
            //Note: If both cars foul only the first foul light will illuminate.
            if (Foul_Flag[0]==1) Win_Flag[1]=true;  //Lane2 is winner if Lane1 fouled first
            if (Foul_Flag[1]==1) Win_Flag[0]=true;  //Lane1 is winner if Lane2 fouled first
          }
        }
        if (Etime1320[0]>0.0 && Etime1320[1]==0.0) { //If only Lane1 finished
          if (Foul_Flag[0]!=1) Win_Flag[0]=true;    //Lane1 is winner if Lane1 not the first to foul
        }
        if (Etime1320[0]==0.0 && Etime1320[1]>0.0) { //If only Lane2 finished
          if (Foul_Flag[1]!=1) Win_Flag[1]=true;    //Lane2 is winner if Lane2 not the first to foul
        }
      }
      if (Lane_Usage<3) {                           //Do if only one racer (Lane1 or Lane2)
        i = Lane_Usage - 1;                         //Set i as the lane array index
        if (Etime1320[i]>0.0 && Foul_Flag[i]==0) { //If Lane finished and didn't foul
          Win_Flag[i]=true;                         //Set Lane as the winner by default
        }
      }
    }
    //Determine winner for handicap race
    if (Hdcap_Flag) {                               //Do if a Handicap race
      if (Etime1320[0]>0.0 && Etime1320[1]>0.0) { //If both lanes finished
        if (Etime1320[0]<float(Dialin1/1000.0)) Breakout_Flag[0] = true; //Set Lane1 Breakout flag
        if (Etime1320[1]<float(Dialin2/1000.0)) Breakout_Flag[1] = true; //Set Lane2 Breakout flag
        if (Foul_Flag[0]==0 && Foul_Flag[1]==0) {   //If both lanes didn't foul
          if (!Breakout_Flag[0] && !Breakout_Flag[1]) { //If both lanes did not breakout
            if (T1320_time[0]<T1320_time[1]) {      //Did Lane1 cross finish line first?
              Win_Flag[0]=true;                     //Lane1 is winner
            }
            if (T1320_time[1]<T1320_time[0]) {      //Did Lane2 cross finish line first?
              Win_Flag[1]=true;                     //Lane2 is winner
            }
            if (T1320_time[0]==T1320_time[1]) {     //Did both lanes cross at same time?
              //Draw: Determine winner by who went fastest
              if (Speed[0]>Speed[1]) Win_Flag[0]=true; //Lane1 is winner
              if (Speed[1]>Speed[0]) Win_Flag[1]=true; //Lane2 is winner
            }
          }
          if (Breakout_Flag[0] && !Breakout_Flag[1]) { //Did only Lane1 breakout?
            Win_Flag[1]=true;                       //Lane2 is winner
          }
          if (Breakout_Flag[1] && !Breakout_Flag[0]) { //Did only Lane2 breakout?
            Win_Flag[0]=true;                       //Lane1 is winner
          }
          if (Breakout_Flag[0] && Breakout_Flag[1]) { //If both lanes breakout (Double Breakout)
            Margin[0] = float(Dialin1/1000.0)-Etime1320[0]; //Determine Lane1 breakout margin
            Margin[1] = float(Dialin2/1000.0)-Etime1320[1]; //Determine Lane2 breakout margin
            if (Margin[0] < Margin[1]) {            //Is Lane1 margin < Lane2 margin?
              Win_Flag[0]=true;                     //Lane1 is winner
            }
            if (Margin[1] < Margin[0]) {            //Is Lane2 margin < Lane1 margin?
              Win_Flag[1]=true;                     //Lane2 is winner
            }
            if (Margin[0] == Margin[1]) {           //Is Lane1 margin = Lane2 margin?
              if (T1320_time[0]<T1320_time[1]) {    //Did Lane1 cross finish line first?
                Win_Flag[0]=true;                   //Lane1 is winner
              }
              if (T1320_time[1]<T1320_time[0]) {    //Did Lane2 cross finish line first?
                Win_Flag[1]=true;                   //Lane2 is winner
              }
              if (T1320_time[0]==T1320_time[1]) {   //Did both lanes cross at same time?
                //DRAW - Determine winner by who went fastest
                if (Speed[0]>Speed[1]) Win_Flag[0]=true; //Lane1 is winner
                if (Speed[1]>Speed[0]) Win_Flag[1]=true; //Lane2 is winner
              }
            }
          } 
        }
        if (Foul_Flag[0]==1 && Foul_Flag[1]==0) {   //If Lane1 only fouled (Foul - trumps a Breakout)
          Win_Flag[1]=true;                         //Lane2 is winner
          Win_Flag[0]=false;                        //Lane1 is loser
        }
        if (Foul_Flag[0]==0 && Foul_Flag[1]==1) {   //If Lane2 only fouled (Foul - trumps a Breakout)
          Win_Flag[0]=true;                         //Lane1 is winner
          Win_Flag[1]=false;                        //Lane2 is loser
        }
        if (Foul_Flag[0]>0 && Foul_Flag[1]>0) {     //If both lanes fouled
          //The one who fouled first is the loser
          if (Foul_Flag[0]==1) {                    //If Lane1 fouled first
            Win_Flag[1]=true;                       //Lane2 is winner (Lane1 fouled)
            Win_Flag[0]=false;                      //Lane1 is loser
          }
          if (Foul_Flag[1]==1) {                    //If Lane2 fouled first
            Win_Flag[0]=true;                       //Lane1 is winner (Lane2 fouled)
            Win_Flag[1]=false;                      //Lane2 is loser
          }
        }
      }
      if (T1320_time[0]>0 && T1320_time[1]==0) {    //If only Lane1 finished
        if (Foul_Flag[0]!=1) {                      //If Lane1 not the first to foul
          Win_Flag[0]=true;                         //Lane1 is winner
        }
      }
      if (T1320_time[0]==0 && T1320_time[1]>0) {    //If only Lane2 finished
          if (Foul_Flag[1]!=1) {                    //If Lane2 not the first to foul
            Win_Flag[1]=true;                       //Lane2 is winner
          }
      }
      if (T1320_time[0]==0 && T1320_time[1]==0) {   //No winner if neither lane finished
        Win_Flag[0]=false;
        Win_Flag[1]=false;
      }
    }
    //Set blinking light for winning lane
    if (Win_Flag[0]) {
      L1blink_Flag = true;                          //Set Lane1 blink
      blinkTime = Clk_Time + 15000;                 //Set 15 second blink time
    }
    if (Win_Flag[1]) {
      L2blink_Flag = true;                          //Set Lane2 blink
      blinkTime = Clk_Time + 15000;                 //Set 15 second blink time
    }
    
    //Send Race Results To PC as one long string for each lane
    //Format: Lanex: y z aaaa.aaa bbbb.bbb cccc.ccc dddd.ddd eeee.eee w
    //Where: x=lane#, y=Foul flag, z=Breakout flag, aaaa.aaa=RT, bbbb.bbb=60Ft ET, cccc.ccc=660Ft ET, 
    //       dddd.ddd=1320Ft ET, eeee.eee=1320Ft Speed, w=Win flag
    for (i=0; i<2; i++) {                           //Loop through all lanes
      if (i==0) Str1 = "Lane1: ";                   //Set Lane1 header if Lane1
      if (i==1) Str1 = "Lane2: ";                   //Set Lane2 header if Lane2
      if (Foul_Flag[i]==0) Str1 = Str1 + "0 ";      //Set lane foul status (0, 1 or 2)
      if (Foul_Flag[i]==1) Str1 = Str1 + "1 ";      //Set lane foul status (0, 1 or 2)
      if (Foul_Flag[i]==2) Str1 = Str1 + "2 ";      //Set lane foul status (0, 1 or 2)
      if (Breakout_Flag[i]) Str1 = Str1 + "1 ";     //Set lane Breakout status
        else Str1 = Str1 + "0 ";
      if (React_time[i]>0) {                        //Check to see if Lane reaction time is valid
        Time = Rtime[i] + 1000.0;                   //Add 1000 to ensure number always has XXXX.XXX format
        Str1 = Str1 + String(Time,3);               //Set lane reaction time trimmed to 3 places
      }
      else Str1 = Str1 + "9999.999";                //Set all nines if time is invalid
      Str1 = Str1 + " ";                            //Add a space to separate numbers
      if (React_time[i]>0 && T060_time[i]>React_time[i]) { //Check to see if lane 60 Ft time is valid
        Time = Etime060[i] + 1000.0;                //Add 1000 to ensure number always has XXXX.XXX format
        Str1 = Str1 + String(Time,3);               //Set lane 6 ft time trimmed to 3 places
      }
      else Str1 = Str1 + "9999.999";                //Set all nines if time is invalid
      Str1 = Str1 + " ";                            //Add a space to separate numbers
      if (React_time[i]>0 && T660_time[i]>React_time[i]) { //Check to see if lane 660 Ft time is valid
        Time = Etime660[i] + 1000.0;                //Add 1000 to ensure number always has XXXX.XXX format
        Str1 = Str1 + String(Time,3);               //Set lane 660 ft time trimmed to 3 places
      }
      else Str1 = Str1 + "9999.999";                //Set all nines if time is invalid
      Str1 = Str1 + " ";                            //Add a space to separate numbers
      if (React_time[i]>0 && T1320_time[i]>React_time[i]) { //Check to see if lane 1320 Ft time is valid
        Time = Etime1320[i] + 1000.0;               //Add 1000 to ensure number always has XXXX.XXX format
        Str1 = Str1 + String(Time,3);               //Set lane 1320 ft time trimmed to 3 places
      }
      else Str1 = Str1 + "9999.999";                //Set all nines if time is invalid
      Str1 = Str1 + " ";                            //Add a space to separate numbers
      if (Speed[i]>0.0) {                           //Check to see if 1/4 mile speed is valid
        Temp = Speed[i] + 1000.0;                   //Add 1000 to ensure number always has XXXX.XXX format
        Str1 = Str1 + String(Temp,3);               //Set lane 1/4 mile speed trimmed to 3 places
      }
      else Str1 = Str1 + "9999.999";                //Set all nines if speed is invalid
      Str1 = Str1 + " ";                            //Add a space to separate numbers
      Str1 = Str1 + String(Win_Flag[i]);            //Set lane win status
      Serial.println(Str1);                         //Send string to PC
      delay(250);                                   //Wait a while to allow PC to process serial data
    }
    Serial.println(SMsg5);                          //Send Finish state message to PC
    trackstate = NotReady;                          //Set track state to Not Ready
  }
  
  //Blink Green "Go" Indicator to show winner
  if (L1blink_Flag) {                               //Check if Lane1 blink flag is set
    Xtime = Clk_Time & 512;                         //Mask time with binary bit that toggles every 512 ms
    if (Xtime==512) {
      digitalWrite(XtreeGrn1,LOW);                  //Turn on Lane1 Green Go Indicator
      if (WinLight_Enable) digitalWrite(WinLight1,LOW); //Turn on Lane1 Win Indicator
    }
    else {
      digitalWrite(XtreeGrn1,HIGH);                 //Turn off Lane1 Green Go Indicator
      if (Clk_Time>blinkTime) {
        L1blink_Flag=false;                         //Force end if blink time expired
        digitalWrite(WinLight1,HIGH);               //Turn off Lane1 Win Indicator
      }
    }
  }
  if (L2blink_Flag) {                               //Check if Lane2 blink flag is set
    Xtime = Clk_Time & 512;                         //Mask time with binary bit that toggles every 512 ms
    if (Xtime==512) {
      digitalWrite(XtreeGrn2,LOW);                  //Turn on Lane2 Green Go Indicator
      if (WinLight_Enable) digitalWrite(WinLight2,LOW); //Turn on Lane2 Win Indicator
    }
    else {
      digitalWrite(XtreeGrn2,HIGH);                 //Turn off Lane2 Green Go Indicator
      if (Clk_Time>blinkTime) {
        L2blink_Flag=false;                         //Force end if blink time expired
        digitalWrite(WinLight2,HIGH);               //Turn off Lane2 Win Indicator
      }
    }
  }

  //FINISHED STATE
  //Manual pushbutton reset routine. Pushbutton must be held for 0.25 seconds to cause a reset
  if (trackstate==Finished || trackstate==NotReady || trackstate==Ready) { //If not staged or racing
    while(digitalRead(hand_sw)==0) {                //Do while hand switch is pressed
      if (millis() > (Clk_Time + 250)) {            //Has switch been held for 1/4 second?
        Serial.println(SMsg8);                      //Send reset request message to PC
        timer_reset();                              //Call timer reset routine
        break;                                      //exit while loop
      }
      delay(50);                                    //Wait awhile before we loop again
    }
  }
}                                                   //Loop back

//=============================== SUBROUTINES & FUNCTIONS ===============================
//Subroutine: Initialize Racing Variables
void initialize() {  
  for (int n=0; n<2; n++) {                   //Loop through both lanes
    React_time[n] = 0;                        // and reset everything
    T060_time[n] = 0;
    T660_time[n] = 0;
    T1254_time[n] = 0;
    T1320_time[n] = 0;
    Rtime[n] = 0.0;
    Etime060[n] = 0.0;
    Etime660[n] = 0.0;
    Etime1320[n] = 0.0;
    Speed[n] = 0.0;
    Otime[n] = 0.0;
  }
  Stg1_Flag = false;                          //Reset Lane1 stage sense flag
  Stg2_Flag = false;                          //Reset Lane2 stage sense flag
  Grd1_Flag = false;                          //Reset Lane1 guard sense flag
  Grd2_Flag = false;                          //Reset Lane2 guard sense flag
  T060L1_Flag = false;                        //Reset Lane1 60 Ft sense flag
  T060L2_Flag = false;                        //Reset Lane2 60 Ft sense flag
  T660L1_Flag = false;                        //Reset Lane1 660 Ft sense flag
  T660L2_Flag = false;                        //Reset Lane2 660 Ft sense flag
  T1254L1_Flag = false;                       //Reset Lane1 1254 Ft sense flag
  T1254L2_Flag = false;                       //Reset Lane2 1254 Ft sense flag
  T1320L1_Flag = false;                       //Reset Lane1 1320 Ft sense flag
  T1320L2_Flag = false;                       //Reset Lane2 1320 Ft sense flag
  Dialin1 = 0;                                //Reset Lane1 dial-in time to 0
  Dialin2 = 0;                                //Reset Lane2 dial-in time to 0
  Serial.flush();                             //Clear the serial input buffer
  return;
}

//Subroutine: Reset Timer
void timer_reset() {
  Foul_Flag[0] = 0;                           //Reset Lane1 foul flag
  Foul_Flag[1] = 0;                           //Reset Lane2 foul flag
  Win_Flag[0] = false;                        //Set Lane1 win flag to false
  Win_Flag[1] = false;                        //Set Lane2 win flag to false
  Breakout_Flag[0] = false;                   //Set Lane1 breakout flag to false
  Breakout_Flag[1] = false;                   //Set Lane2 breakout flag to false
  L1blink_Flag = false;                       //Set Lane1 blink command to false (off)
  L2blink_Flag = false;                       //Set Lane2 blink command to false (off)
  blinkTime = 0;                              //Reset blink time
  Xtime = 0;                                  //Clear time variable
  Turnoff_All_Lights();                       //Call subroutine to turn off Xmas Tree lights
  Check_Track();                              //Call Check Track subroutine
  if (Track_ready){                           //and only do a reset if Track is ready
    initialize();                             //Call initialize function & reset everything
    trackstate = Ready;                       //Set race state to Ready
    Ln1_State = Ready;                        //Set Lane1 racestate to ready
    Ln2_State = Ready;                        //Set Lane2 racestate to ready
    if (TopBulbLock_Mode) TBL_Enable = true;  //Enable Top Bulb Lock if Top Bulb Lock mode is selected
    Serial.println(SMsg1);                    //Send Ready state message to PC
  } 
  else {                                      //Do this if track is not ready
    trackstate = NotReady;                    //Set race state to Not Ready
    if (!Lane1_ready) Ln1_State = NotReady;   //Set Lane1 racestate to not ready
    if (!Lane2_ready) Ln2_State = NotReady;   //Set Lane2 racestate to not ready
    Serial.println(SMsg0);                    //Send Not Ready state message to PC
  }
}

//Subroutine: Check Track - Sets 'Track_ready' to true or false
void Check_Track() {
  Track_ready = true;                         //Default track ready status to true
  Lane1_ready = true;                         //Default Lane1 sensor ready status to true
  Lane2_ready = true;                         //Default Lane2 sensor ready status to true
  error_codes = "";                           //Clear bad sensor string
  //Check Lane1 Sensors
  if (digitalRead(PStg_Sense1)==Trip) {       //Check if a Lane1 Pre-Stage sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " PStg1";     //Build error string identifying obstructed sensors
  }
  if (digitalRead(Stg_Sense1)==Trip) {        //Check if a Lane1 Stage sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " Stg1";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(Grd_Sense1)==Trip) {        //Check if a Lane1 Guard sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " Grd1";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(T060_Sense1)==Trip) {       //Check if a Lane1 60 Ft track sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " 60ft1";     //Build error string identifying obstructed track sensors
  }
  if (digitalRead(T660_Sense1)==Trip) {       //Check if a Lane1 660 Ft track sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " 660ft1";    //Build error string identifying obstructed track sensors
  }
  if (digitalRead(T1254_Sense1)==Trip) {      //Check if a Lane1 1254 Ft track sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " 1254ft1";   //Build error string identifying obstructed lane sensors
  }
  if (digitalRead(T1320_Sense1)==Trip) {      //Check if a Lane1 1320 Ft track sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " 1320ft1";   //Build error string identifying obstructed lane sensors
  }
  //Check Lane2 Sensors
  if (digitalRead(PStg_Sense2)==Trip) {       //Check if a Lane2 Pre-Stage sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " PStg2";     //Build error string identifying obstructed sensors
  }
  if (digitalRead(Stg_Sense2)==Trip) {        //Check if a Lane2 Stage sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " Stg2";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(Grd_Sense2)==Trip) {        //Check if a Lane2 Guard sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " Grd2";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(T060_Sense2)==Trip) {       //Check if a Lane2 60 Ft track sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " 60ft2";     //Build error string identifying obstructed track sensors
  }
  if (digitalRead(T660_Sense2)==Trip) {       //Check if a Lane2 660 Ft track sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " 660ft2";    //Build error string identifying obstructed track sensors
  }
  if (digitalRead(T1254_Sense2)==Trip) {      //Check if a Lane2 1254 Ft track sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " 1254ft2";   //Build error string identifying obstructed lane sensors
  }
  if (digitalRead(T1320_Sense2)==Trip) {      //Check if a Lane2 1320 Ft track sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " 1320ft2";   //Build error string identifying obstructed lane sensors
  }
  if (!Lane1_ready || !Lane2_ready) {         //Check if either lane is not ready
    Serial.println(SMsg7 + error_codes);      //If so, send error message to PC
    Track_ready = false;                      //Set track ready status to false
  }
  for (i=0; i<2; i++) {                       //Do lane ready/notready blink routine
    if (!Lane1_ready) digitalWrite(XtreeDsq1,LOW); //Blink Lane1 red light if Lane1 not ready
      else digitalWrite(XtreeGrn1,LOW);            //Blink Lane1 green light if Lane1 ready
    if (!Lane2_ready) digitalWrite(XtreeDsq2,LOW); //Blink Lane2 red light if Lane2 not ready
      else digitalWrite(XtreeGrn2,LOW);            //Blink Lane2 green light if Lane2 ready
    delay(75);
    digitalWrite(XtreeDsq1,HIGH);
    digitalWrite(XtreeDsq2,HIGH);
    digitalWrite(XtreeGrn1,HIGH);
    digitalWrite(XtreeGrn2,HIGH);
    delay(200);
  }
}

//Subroutine - Xmas Tree lamp test
void Lamptest() {
  digitalWrite(XtreePstg1,LOW);               //Turn on Lane1 Pre-Stage Indicator
  digitalWrite(XtreePstg2,LOW);               //Turn on Lane2 Pre-Stage Indicator
  delay(500);
  digitalWrite(XtreePstg1,HIGH);              //Turn off Lane1 Pre-Stage Indicator
  digitalWrite(XtreePstg2,HIGH);              //Turn off Lane2 Pre-Stage Indicator
  digitalWrite(XtreeStg1,LOW);                //Turn on Lane1 Stage Indicator
  digitalWrite(XtreeStg2,LOW);                //Turn on Lane2 Stage Indicator
  delay(500);
  digitalWrite(XtreeStg1,HIGH);               //Turn off Lane1 Stage Indicator
  digitalWrite(XtreeStg2,HIGH);               //Turn off Lane2 Stage Indicator
  digitalWrite(XtreeSeq1a,LOW);               //Turn on Lane1 Amber Indicator #1
  digitalWrite(XtreeSeq2a,LOW);               //Turn on Lane2 Amber Indicator #1
  delay(500);
  digitalWrite(XtreeSeq1a,HIGH);              //Turn off Lane1 Amber Indicator #1
  digitalWrite(XtreeSeq2a,HIGH);              //Turn off Lane2 Amber Indicator #1
  digitalWrite(XtreeSeq1b,LOW);               //Turn on Lane1 Amber Indicator #2
  digitalWrite(XtreeSeq2b,LOW);               //Turn on Lane2 Amber Indicator #2
  delay(500);
  digitalWrite(XtreeSeq1b,HIGH);              //Turn off Lane1 Amber Indicator #2
  digitalWrite(XtreeSeq2b,HIGH);              //Turn off Lane2 Amber Indicator #2
  digitalWrite(XtreeSeq1c,LOW);               //Turn on Lane1 Amber Indicator #3
  digitalWrite(XtreeSeq2c,LOW);               //Turn on Lane2 Amber Indicator #3
  delay(500);
  digitalWrite(XtreeSeq1c,HIGH);              //Turn off Lane1 Amber Indicator #3
  digitalWrite(XtreeSeq2c,HIGH);              //Turn off Lane2 Amber Indicator #3
  digitalWrite(XtreeGrn1,LOW);                //Turn on Lane1 Green Go Indicator
  digitalWrite(XtreeGrn2,LOW);                //Turn on Lane2 Green Go Indicator
  delay(500);
  digitalWrite(XtreeGrn1,HIGH);               //Turn off Lane1 Green Go Indicator
  digitalWrite(XtreeGrn2,HIGH);               //Turn off Lane2 Green Go Indicator
  digitalWrite(XtreeDsq1,LOW);                //Turn on Lane1 Disqualification Indicator
  digitalWrite(XtreeDsq2,LOW);                //Turn on Lane2 Disqualification Indicator
  delay(500);
  digitalWrite(XtreeDsq1,HIGH);               //Turn off Lane1 Disqualification Indicator
  digitalWrite(XtreeDsq2,HIGH);               //Turn off Lane2 Disqualification Indicator
  if (WinLight_Enable) {                      //Do if Win Light feature is enabled
    digitalWrite(WinLight1,LOW);              //Turn on Lane1 Win Indicator
    digitalWrite(WinLight2,LOW);              //Turn on Lane2 Win Indicator
    delay(500);
    digitalWrite(WinLight1,HIGH);             //Turn off Lane1 Win Indicator
    digitalWrite(WinLight2,HIGH);             //Turn off Lane2 Win Indicator
  }
}

//Subroutine - Turn off all Xmas Tree lights
void Turnoff_All_Lights() {
  digitalWrite(XtreePstg1,HIGH);              //Turn off Lane1 white Pre-tage Indicator
  digitalWrite(XtreeStg1,HIGH);               //Turn off Lane1 white Stage Indicator
  digitalWrite(XtreeSeq1a,HIGH);              //Turn off Lane1 Amber Indicator #1
  digitalWrite(XtreeSeq1b,HIGH);              //Turn off Lane1 Amber Indicator #2
  digitalWrite(XtreeSeq1c,HIGH);              //Turn off Lane1 Amber Indicator #3
  digitalWrite(XtreeGrn1,HIGH);               //Turn off Lane1 Green Go Indicator
  digitalWrite(XtreeDsq1,HIGH);               //Turn off Lane1 Red Disq. Indicator
  digitalWrite(XtreePstg2,HIGH);              //Turn off Lane2 white Pre-tage Indicator
  digitalWrite(XtreeStg2,HIGH);               //Turn off Lane2 white Stage Indicator
  digitalWrite(XtreeSeq2a,HIGH);              //Turn off Lane2 Amber Indicator #1
  digitalWrite(XtreeSeq2b,HIGH);              //Turn off Lane2 Amber Indicator #2
  digitalWrite(XtreeSeq2c,HIGH);              //Turn off Lane2 Amber Indicator #3
  digitalWrite(XtreeGrn2,HIGH);               //Turn off Lane2 Green Go Indicator
  digitalWrite(XtreeDsq2,HIGH);               //Turn off Lane2 Red Disq. Indicator
  digitalWrite(WinLight1,HIGH);               //Turn off Lane1 Win Indicator
  digitalWrite(WinLight2,HIGH);               //Turn off Lane2 Win Indicator
}

//Subroutine - Set Xmas tree dial-In delays
void Set_Dialin_Delays() {
  Hdcap_Flag = false;                         //Preset handicap racing flag to false
  Dialin_Dly1 = 0;                            //Preset Lane1 Xmas tree time delay to zero
  Dialin_Dly2 = 0;                            //Preset Lane2 Xmas tree time delay to zero
  if (Dialin1>0 && Dialin2>0) {               //Do if dial-in times are being used?
    Hdcap_Flag = true;                        //Set handicap flag to true if dial-in times used
    if (Dialin2 < Dialin1) {                  //Do if Lane2 dial-in time < Lane1 dial-in time
      Dialin_Dly2 = Dialin1 - Dialin2;        //Calculate Lane2 Xmas tree sequence time delay
    }
    if (Dialin1 < Dialin2) {                  //Do if Lane1 dial-in time < Lane2 dial-in time
      Dialin_Dly1 = Dialin2 - Dialin1;        //Calculate Lane1 Xmas tree sequence time delay
    }
  }
  Ln1_Seq_State = 0;                          //Initialize Lane1 Sequence State count to 0
  Ln2_Seq_State = 0;                          //Initialize Lane2 Sequence State count to 0
}

void Calculate_Times(int t) {
  if (React_time[t]>0) {                      //Check to see if Lane reaction time is valid
    if (Foul_Flag[t]>0) {                     //Did car leave early? (negative reaction time)
      Rtime[t] = float(Start_time[t]-React_time[t])/1000.0; //Calc reaction time in seconds
    }
    else {                                    //Car left after green light (positive reaction time)
      Rtime[t] = float(React_time[t]-Start_time[t])/1000.0; //Calc reaction time in seconds
    }
    if (T060_time[t]>React_time[t]) {                       //Is 60 Ft time valid?
      Etime060[t] = float((T060_time[t]-React_time[t])/1000.0); //Calculate 60 Ft ET (Seconds)
    }
    if (T660_time[t]>React_time[t]) {                       //Is 660 Ft (1/8 mile finish) time valid?
      Etime660[t] = float((T660_time[t]-React_time[t])/1000.0); //Calculate 660 Ft ET (Seconds)
    }
    if (T1320_time[t]>React_time[t]) {                      //Is 1/4 mile finish time valid?
      Etime1320[t] = float((T1320_time[t]-React_time[t])/1000.0); //Calculate 1/4 mile ET (Seconds)
      if (Foul_Flag[t]>0) {                                 //Did car leave early? (negative reaction time)
        Otime[t] = Etime1320[t];                            //Set race Overall time to 1/4 mile ET
      }
      else {                                                //Car did not leave early
        Otime[t] = Rtime[t] + Etime1320[t];                 //Set race Overall time to 1/4 mile RT + ET
      }
      if (T1254_time[t]>0 && T1320_time[t]>T1254_time[t]) { //Is 1/4 mile trap time valid?
        Xtime = T1320_time[t]-T1254_time[t];                //Calc time between 1/4 mile trap & finish (milliseconds)
        if (kmh_Flag) Speed[t] = float(72420.5/Xtime);      //Calculate 1/4 mile lane speed in Km/h
        else Speed[t] = float(45000.0/Xtime);               //Calculate 1/4 mile lane speed in MPH
      }
    }
  }
}
